#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.25 01:00:00                  #
# ================================================== #

from PySide6.QtCore import Slot, QTimer

from pygpt_net.core.render.base import BaseRenderer
from pygpt_net.core.render.markdown.renderer import Renderer as MarkdownRenderer
from pygpt_net.core.render.plain.renderer import Renderer as PlainTextRenderer
from pygpt_net.core.render.web.renderer import Renderer as WebRenderer
from pygpt_net.core.text.utils import output_html2text, output_clean_html
from pygpt_net.item.ctx import CtxItem


class Render:
    def __init__(self, window=None):
        """
        Render controller

        :param window: Window instance
        """
        self.window = window
        self.markdown_renderer = MarkdownRenderer(window)
        self.plaintext_renderer = PlainTextRenderer(window)
        self.web_renderer = WebRenderer(window)
        self.engine = None
        self.scroll = 0

    def setup(self):
        """Setup render"""
        self.engine = self.window.core.config.get("render.engine")
        if self.engine == "web":
            self.connect_signals()

    def connect_signals(self):
        """Connect signals"""
        signals = self.web_renderer.get_output_node().signals
        signals.save_as.connect(self.handle_save_as)
        signals.audio_read.connect(self.handle_audio_read)

    def get_engine(self) -> str:
        """
        Get current render engine

        :return: engine name
        """
        return self.engine

    def get_renderer(self) -> BaseRenderer:
        """
        Get current renderer instance

        :return: Renderer instance
        """
        # get selected renderer
        if self.window.core.config.get('render.plain'):
            return self.plaintext_renderer
        else:
            if self.engine == "web":
                return self.web_renderer
            else:
                return self.markdown_renderer

    def switch(self, live: bool = True):
        """
        Switch renderer (markdown <==> plain text)

        :param live: True if live update
        """
        plain = self.window.core.config.get('render.plain')
        if plain:
            self.window.controller.theme.markdown.clear()
            if self.window.ui.nodes['output'] is not None:
                if self.window.ui.nodes['output'].isVisible():
                    self.window.ui.nodes['output'].setVisible(False)
                    self.window.ui.nodes['output_plain'].setVisible(True)
        else:
            self.window.controller.ctx.refresh()  # TODO: move to on_switch
            self.window.controller.theme.markdown.update(force=True)
            if self.window.ui.nodes['output_plain'].isVisible():
                if self.window.ui.nodes['output'] is not None:
                    self.window.ui.nodes['output'].setVisible(True)
                    self.window.ui.nodes['output_plain'].setVisible(False)

    def begin(self, stream: bool = False):
        """
        Render begin

        :param stream: True if it is a stream
        """
        self.get_renderer().begin(stream)
        self.update()

    def end(self, stream: bool = False):
        """
        Render end

        :param stream: True if it is a stream
        """
        self.get_renderer().end(stream)
        self.update()

    def end_extra(self, stream: bool = False):
        """
        Render end extra

        :param stream: True if it is a stream
        """
        self.get_renderer().end_extra(stream)
        self.update()

    def stream_begin(self):
        """Render stream begin"""
        self.get_renderer().stream_begin()
        self.update()

    def stream_end(self):
        """Render stream end"""
        self.get_renderer().stream_end()
        self.update()

    def clear_output(self):
        """Clear output"""
        self.get_renderer().clear_output()
        self.update()

    def clear_input(self):
        """Clear input"""
        self.get_renderer().clear_input()

    def reset(self):
        """Reset"""
        self.get_renderer().reset()
        self.update()

    def reload(self):
        """Reload output"""
        self.get_renderer().reload()
        self.update()

    def append_context(self, items: list, clear: bool = True):
        """
        Append all context to output

        :param items: Context items
        :param clear: True if clear all output before append
        """
        self.get_renderer().append_context(items, clear)
        self.update()

    def append_input(self, item: CtxItem):
        """
        Append text input to output

        :param item: context item
        """
        self.get_renderer().append_input(item)
        self.update()

    def append_output(self, item: CtxItem):
        """
        Append text output to output

        :param item: context item
        """
        self.get_renderer().append_output(item)
        self.update()

    def append_extra(self, item: CtxItem, footer: bool = False):
        """
        Append extra data (images, files, etc.) to output

        :param item: context item
        :param footer: True if it is a footer
        """
        self.get_renderer().append_extra(item, footer)
        self.update()

    def append_chunk(self, item: CtxItem, text_chunk: str, begin: bool = False):
        """
        Append output stream chunk to output

        :param item: context item
        :param text_chunk: text chunk
        :param begin: if it is the beginning of the stream
        """
        self.get_renderer().append_chunk(item, text_chunk, begin)
        self.update()

    def on_enable_edit(self, live: bool = True):
        """
        On enable edit icons

        :param live: True if live update
        """
        self.get_renderer().on_enable_edit(live)
        self.update()

    def on_disable_edit(self, live: bool = True):
        """
        On disable edit icons

        :param live: True if live update
        """
        self.get_renderer().on_disable_edit(live)
        self.update()

    def on_enable_timestamp(self, live: bool = True):
        """
        On enable timestamp

        :param live: True if live update
        """
        self.get_renderer().on_enable_timestamp(live)
        self.update()

    def on_disable_timestamp(self, live: bool = True):
        """
        On disable timestamp

        :param live: True if live update
        """
        self.get_renderer().on_disable_timestamp(live)
        self.update()

    def remove_item(self, id: int):
        """
        Remove item from output

        :param id: context item ID
        """
        self.get_renderer().remove_item(id)
        self.update()

    def remove_items_from(self, id: int):
        """
        Remove item from output

        :param id: context item ID
        """
        self.get_renderer().remove_items_from(id)
        self.update()

    def on_edit_submit(self, id: int):
        """
        On edit submit

        :param id: context item ID
        """
        self.get_renderer().on_edit_submit(id)
        self.update()

    def on_remove_submit(self, id: int):
        """
        On remove submit

        :param id: context item ID
        """
        self.get_renderer().on_remove_submit(id)
        self.update()

    def on_reply_submit(self, id: int):
        """
        On regenerate submit

        :param id: context item ID
        """
        self.get_renderer().on_reply_submit(id)
        self.update()

    def on_page_loaded(self):
        """On page loaded callback"""
        self.get_renderer().on_page_loaded()
        self.update()

    def on_theme_change(self):
        """On theme change"""
        if self.get_engine() == "web":
            self.web_renderer.on_theme_change()
        elif self.get_engine() == "legacy":
            self.markdown_renderer.on_theme_change()
        self.update()

    def get_scroll_position(self) -> int:
        """
        Get scroll position

        :return: scroll position
        """
        return self.get_renderer().get_scroll_position()

    def set_scroll_position(self, position: int):
        """
        Set scroll position

        :param position: scroll position
        """
        self.get_renderer().set_scroll_position(position)
        self.update()

    def update(self):
        """On update"""
        self.window.ui.nodes['output'].on_update()

    def clear_all(self):
        """Clear all"""
        self.get_renderer().clear_all()
        self.update()

    @Slot(str, str)
    def handle_save_as(self, text: str, type: str = 'txt'):
        """
        Handle save as signal

        :param text: Data to save
        :param type: File type
        """
        if type == 'html':
            text = output_clean_html(text)
        else:
            text = output_html2text(text)
        # fix: QTimer required here to prevent crash if signal emitted from WebEngine window
        QTimer.singleShot(0, lambda: self.window.controller.chat.common.save_text(text, type))

    @Slot(str)
    def handle_audio_read(self, text: str):
        """
        Handle audio read signal

        :param text: Text to read
        """
        self.window.controller.audio.read_text(text)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.06 22:00:00                  #
# ================================================== #

import os

from packaging.version import parse as parse_version, Version


class Patch:
    def __init__(self, window=None):
        self.window = window

    def execute(self, version: Version) -> bool:
        """
        Migrate config to current app version

        :param version: current app version
        :return: True if migrated
        """
        data = self.window.core.plugins.get_presets()
        current = "0.0.0"
        updated = False
        is_old = False

        # get version of config file
        if '__meta__' in data and 'version' in data['__meta__']:
            current = data['__meta__']['version']
        old = parse_version(current)

        # check if config file is older than current app version
        if old < version:

            # mark as older version
            is_old = True

            # < 0.9.1
            if old < parse_version("2.1.10"):
                print("Migrating presets from < 2.1.10...")
                pass
                updated = True

        # update file
        migrated = False
        if updated:
            data = dict(sorted(data.items()))
            self.window.core.plugins.presets = data
            self.window.core.plugins.save_presets()
            migrated = True

        return migrated



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.26 15:00:00                  #
# ================================================== #

import datetime
import os.path
import shutil

from PySide6.QtGui import QAction, QIcon

from pygpt_net.tools.base import BaseTool
from pygpt_net.tools.media_player.ui.dialogs import VideoPlayer
from PySide6.QtWidgets import QMessageBox, QFileDialog

from pygpt_net.utils import trans


class MediaPlayer(BaseTool):
    def __init__(self, *args, **kwargs):
        """
        Media player

        :param window: Window instance
        """
        super(MediaPlayer, self).__init__(*args, **kwargs)
        self.id = "player"
        self.opened = False
        self.dialog = None

    def setup(self):
        """Setup media player"""
        if self.window.core.config.has("video.player.path"):
            path = self.window.core.config.get("video.player.path")
            if path:
                path = self.window.core.filesystem.to_workdir(path)
                if os.path.exists(path):
                    self.window.video_player.set_path(path)
        if self.window.core.config.has("video.player.volume"):
            self.window.video_player.adjust_volume(self.window.core.config.get("video.player.volume"))
        if self.window.core.config.has("video.player.volume.mute"):
            self.window.video_player.set_muted(self.window.core.config.get("video.player.volume.mute"))
        self.window.video_player.update()  # update player volume, slider, etc.

    def update(self):
        """Update menu"""
        self.update_menu()

    def update_menu(self):
        """Update menu"""
        """
        if self.opened:
            self.window.ui.menu['tools.media.player'].setChecked(True)
        else:
            self.window.ui.menu['tools.media.player'].setChecked(False)
        """

    def store_path(self, path: str):
        """
        Store current video path

        :param path: video file path
        """
        path = self.window.core.filesystem.make_local(path)
        self.window.core.config.set("video.player.path", path)

    def play(self, path: str):
        """
        Open and play video file

        :param path: video file path
        """
        self.window.ui.dialogs.open('video_player', width=800, height=600)
        self.window.video_player.force_resize()
        self.opened = True
        self.update()
        self.window.video_player.open(path)

    def open_file(self):
        """Open video file"""
        self.window.video_player.open_file()

    def grab_frame(self) -> str:
        """
        Grab frame

        :return: frame path
        """
        now = datetime.datetime.now()
        dt = now.strftime("%Y-%m-%d_%H-%M-%S")
        name = 'cap-' + dt
        path = os.path.join(self.window.controller.painter.common.get_capture_dir(), name + '.png')
        # TODO: implement grab screenshot
        return path

    def save_as_file(self):
        """Save video as file"""
        path = self.window.video_player.path
        if not path:
            QMessageBox.warning(self.window.video_player, "Save Error", "No video loaded.")
            return
        save_path, _ = QFileDialog.getSaveFileName(
            self.window.video_player,
            "Save video As",
            path,
            "Video Files (*.mp4 *.avi *.mkv)",
        )
        if save_path and save_path != path:
            try:
                shutil.copy2(path, save_path)
                QMessageBox.information(
                    self.window.video_player,
                    "OK",
                    f"Video saved to: {save_path}"
                )
            except Exception as e:
                QMessageBox.critical(
                    self.window.video_player,
                    "Error",
                    f"An error occurred while saving the video: {e}"
                )
    def open(self):
        """Open player window"""
        self.window.ui.dialogs.open('video_player', width=800, height=600)
        self.window.video_player.force_resize()
        self.opened = True
        self.update()

    def close(self):
        """Close player window"""
        self.on_close()
        self.window.ui.dialogs.close('video_player')
        self.opened = False
        self.update()

    def on_close(self):
        """On player window close"""
        self.opened = False
        self.update()
        self.window.video_player.on_close()

    def toggle(self):
        """Toggle player window"""
        if self.opened:
            self.close()
        else:
            self.open()

    def show_hide(self, show: bool = True):
        """
        Show/hide player window

        :param show: show/hide
        """
        if show:
            self.open()
        else:
            self.close()

    def setup_menu(self) -> dict:
        """
        Setup main menu

        :return dict with menu actions
        """
        actions = {}
        actions["media.player"] = QAction(
            QIcon(":/icons/video.svg"),
            trans("menu.tools.media.player"),
            self.window,
            checkable=False,
        )
        actions["media.player"].triggered.connect(
            lambda: self.toggle()
        )
        return actions

    def setup_dialogs(self):
        """Setup dialogs (static)"""
        self.dialog = VideoPlayer(self.window)
        self.dialog.setup()

    def get_lang_mappings(self) -> dict:
        """
        Get language mappings

        :return: dict with language mappings
        """
        return {
            'menu.text': {
                'tools.media.player': 'menu.tools.media.player',
            }
        }


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.09 23:00:00                  #
# ================================================== #

from PySide6.QtGui import QStandardItemModel
from PySide6.QtWidgets import QPushButton, QHBoxLayout, QVBoxLayout

from pygpt_net.ui.widget.dialog.profile import ProfileDialog, ProfileEditDialog
from pygpt_net.ui.widget.element.labels import HelpLabel
from pygpt_net.ui.widget.lists.profile import ProfileList
from pygpt_net.utils import trans


class Profile:
    def __init__(self, window=None):
        """
        Profile editor dialog

        :param window: Window instance
        """
        self.window = window
        self.dialog_id = "profile.editor"

    def setup(self):
        """Setup presets editor dialog"""
        self.window.ui.nodes['profile.editor.btn.new'] = \
            QPushButton(trans("dialog.profile.new"))

        self.window.ui.nodes['profile.editor.btn.new'].clicked.connect(
            lambda: self.window.controller.settings.profile.new()
        )

        # set enter key to save button
        self.window.ui.nodes['profile.editor.btn.new'].setAutoDefault(False)

        # footer buttons
        footer = QHBoxLayout()
        footer.addWidget(self.window.ui.nodes['profile.editor.btn.new'])
        # footer.addWidget(self.window.ui.nodes[profile.editor.btn.save'])

        # list
        id = 'profile.list'
        self.window.ui.nodes[id] = ProfileList(self.window, id)
        self.window.ui.models[id] = self.create_model(self.window)
        self.window.ui.nodes[id].setModel(self.window.ui.models[id])

        # get
        data = self.window.controller.settings.profile.get_profiles()

        # update list
        self.update_list(id, data)

        self.window.ui.nodes['profile.editor.tip'] = HelpLabel(trans("dialog.profile.tip"))

        main_layout = QVBoxLayout()
        main_layout.addWidget(self.window.ui.nodes[id])
        main_layout.addWidget(self.window.ui.nodes['profile.editor.tip'])

        layout = QVBoxLayout()
        layout.addLayout(main_layout)  # list
        layout.addLayout(footer)  # bottom buttons (save, new)

        self.window.ui.dialog[self.dialog_id] = ProfileDialog(self.window, self.dialog_id)
        self.window.ui.dialog[self.dialog_id].setLayout(layout)
        self.window.ui.dialog[self.dialog_id].setWindowTitle(trans('dialog.profile.editor'))

    def create_model(self, parent) -> QStandardItemModel:
        """
        Create list model

        :param parent: parent widget
        :return: QStandardItemModel
        """
        return QStandardItemModel(0, 1, parent)

    def update_list(self, id: str, data: dict):
        """
        Update list

        :param id: ID of the list
        :param data: Data to update
        """
        if id not in self.window.ui.models:
            return

        current = self.window.core.config.profile.get_current()
        self.window.ui.models[id].removeRows(0, self.window.ui.models[id].rowCount())
        i = 0
        for n in data:
            self.window.ui.models[id].insertRow(i)
            name = data[n]['name']
            if n == current:
                name += " " + trans("profile.current.suffix")
            index = self.window.ui.models[id].index(i, 0)
            self.window.ui.models[id].setData(index, name)
            i += 1

class ProfileEdit:
    def __init__(self, window=None):
        """
        Profile edit dialog

        :param window: Window instance
        """
        self.window = window

    def setup(self):
        """Setup dialog"""
        id = 'profile.item'
        self.window.ui.dialog[id] = ProfileEditDialog(self.window, id)
        self.window.ui.dialog[id].setWindowTitle(trans("dialog.profile.item.editor"))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.08 21:00:00                  #
# ================================================== #

from PySide6.QtCore import QItemSelectionModel
from PySide6.QtWidgets import QTreeView, QAbstractItemView


class BaseList(QTreeView):
    NAME = range(1)  # list of columns

    def __init__(self, window=None, id=None):
        """
        Select menu

        :param window: Window instance
        :param id: input id
        """
        super(BaseList, self).__init__(window)
        self.window = window
        self.id = id
        self.setEditTriggers(QAbstractItemView.NoEditTriggers)
        self.setIndentation(0)
        self.selection_locked = None
        self.selection = None
        self.unlocked = False
        self.clicked.connect(self.click)
        self.header().hide()

    def click(self, val):
        self.window.controller.mode.select(self.id, val.row())
        self.selection = self.selectionModel().selection()

    def lockSelection(self, selected=None, deselected=None):
        if self.selection is not None:
            self.selectionModel().select(self.selection, QItemSelectionModel.Select)

    def backup_selection(self):
        self.selection = self.selectionModel().selection()

    def restore_selection(self):
        if self.selection is not None:
            self.selectionModel().select(self.selection, QItemSelectionModel.Select)

    def mousePressEvent(self, event):
        index = self.indexAt(event.pos())
        if not index.isValid():
            return
        super(BaseList, self).mousePressEvent(event)

    def focusOutEvent(self, event):
        pass

    def selectionCommand(self, index, event=None):
        """
        Selection command
        :param index: Index
        :param event: Event
        """
        # check tmp unlock
        if self.unlocked:
            return super().selectionCommand(index, event)

        if self.selection_locked is not None and self.selection_locked():
            return QItemSelectionModel.NoUpdate
        return super().selectionCommand(index, event)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.17 07:00:00                  #
# ================================================== #

import time
from uuid import uuid4
from packaging.version import Version

from pygpt_net.item.ctx import CtxMeta, CtxItem, CtxGroup
from .patch import Patch
from .storage import Storage
from pygpt_net.provider.core.ctx.base import BaseProvider


class DbSqliteProvider(BaseProvider):
    def __init__(self, window=None):
        super(DbSqliteProvider, self).__init__(window)
        self.window = window
        self.patcher = Patch(window, self)
        self.storage = Storage(window)
        self.id = "db_sqlite"
        self.type = "ctx"

    def attach(self, window):
        self.window = window
        self.storage.attach(window)

    def patch(self, version: Version):
        """
        Patch versions

        :param version: current app version
        """
        return self.patcher.execute(version)

    def create_id(self, meta: CtxMeta) -> int:
        """
        Create ctx ID

        :return: generated ID
        """
        return self.storage.insert_meta(meta)

    def create(self, meta: CtxMeta) -> int:
        """
        Create new ctx and return its ID

        :param meta: CtxMeta
        :return: ctx ID
        """
        ts = int(time.time())
        if meta.uuid is None:
            meta.uuid = str(uuid4())
        if meta.created is None:
            meta.created = ts
        if meta.updated is None:
            meta.updated = ts
        if meta.id is None:
            meta.id = self.create_id(meta)  # insert to DB and get ID
        return meta.id

    def get_meta(
            self,
            search_string: str = None,
            order_by: str = None,
            order_direction: str = None,
            limit: int = None,
            offset: int = None,
            filters: dict = None,
            search_content: bool = False,
    ) -> dict:
        """
        Return dict of ctx meta, TODO: add order, limit, offset, etc.

        :param search_string: search string
        :param order_by: order by column
        :param order_direction: order direction (asc, desc)
        :param limit: limit
        :param offset: offset
        :param filters: filters
        :param search_content: search in content (not only in meta)
        :return: dict of ctx meta
        """
        param_limit = 0
        if limit is not None:
            param_limit = int(limit)

        return self.storage.get_meta(
            search_string=search_string,
            order_by=order_by,
            order_direction=order_direction,
            limit=param_limit,
            offset=offset,
            filters=filters,
            search_content=search_content,
        )

    def get_meta_indexed(self) -> dict:
        """
        Return dict of ctx meta indexed by ID

        :return: dict of ctx meta indexed by ID
        """
        return self.storage.get_meta_indexed()

    def load(self, id: int) -> list:
        """
        Load items for ctx ID

        :param id: ctx ID
        :return: list of ctx items
        """
        return self.storage.get_items(id)

    def get_ctx_count_by_day(
            self, year: int,
            month: int = None,
            day: int = None,
            search_string: str = None,
            filters: dict = None,
            search_content: bool = False,
    ) -> dict:
        """
        Get ctx count by day or by month if only year is provided

        :param year: year
        :param month: month
        :param day: day
        :param search_string: search string (optional)
        :param filters: filters (optional)
        :param search_content: search in content (not only in meta) (optional)
        :return: dict of ctx counters by day or by month if only year is provided
        """
        return self.storage.get_ctx_count_by_day(
            year=year,
            month=month,
            day=day,
            search_string=search_string,
            filters=filters,
            search_content=search_content,
        )

    def append_item(self, meta: CtxMeta, item: CtxItem) -> bool:
        """
        Append item to ctx

        :param meta: ctx meta (CtxMeta)
        :param item: ctx item (CtxItem)
        :return: True if appended
        """
        self.storage.update_meta_ts(meta.id)
        return self.storage.insert_item(meta, item) is not None

    def update_item(self, item: CtxItem) -> bool:
        """
        Update item in ctx

        :param item: ctx item (CtxItem)
        :return: True if updated
        """
        self.storage.update_meta_ts(item.meta_id)
        return self.storage.update_item(item) is not None

    def save(self, id: int, meta: CtxMeta, items: list) -> bool:
        """
        Save ctx

        :param id: ctx ID
        :param meta: CtxMeta
        :param items: list of CtxItem
        :return: True if saved
        """
        if self.storage.update_meta(meta):
            return True  # update only meta, items are appended separately

    def save_all(self, id: int, meta: CtxMeta, items: list) -> bool:
        """
        Save ctx

        :param id: ctx ID
        :param meta: CtxMeta
        :param items: list of CtxItem
        :return: True if saved
        """
        if self.storage.update_meta_all(meta, items):
            return True

    def remove(self, id: int) -> bool:
        """
        Remove ctx (meta + items)

        :param id: ctx meta ID
        :return: True if removed
        """
        return self.storage.delete_meta_by_id(id)

    def remove_item(self, id: int) -> bool:
        """
        Remove ctx item

        :param id: ctx item ID
        :return: True if removed
        """
        return self.storage.delete_item_by_id(id)

    def remove_items_from(self, meta_id: int, item_id: int):
        """
        Remove ctx items from meta_id

        :param meta_id: meta_id
        :param item_id: item_id
        :return: True if removed
        """
        return self.storage.delete_items_from(meta_id, item_id)

    def truncate(self) -> bool:
        """
        Truncate ctx (remove all ctx, meta + items)

        :return: True if truncated
        """
        return self.storage.truncate_all()

    def set_meta_indexed_by_id(self, id: int, ts: int) -> bool:
        """
        Set meta indexed by ID

        :param id: ctx ID
        :param ts: timestamp
        :return: True if set
        """
        return self.storage.set_meta_indexed_by_id(id, ts)

    def update_meta_indexes_by_id(self, id: int, meta: CtxMeta) -> bool:
        """
        Update meta indexes by ID

        :param id: ctx ID
        :param meta: CtxMeta
        :return: True if updated
        """
        return self.storage.update_meta_indexes_by_id(id, meta)

    def update_meta_indexed_by_id(self, id: int) -> bool:
        """
        Update meta indexed timestamp by ID

        :param id: ctx ID
        :return: True if updated
        """
        return self.storage.update_meta_indexed_by_id(id)

    def update_meta_indexed_to_ts(self, ts: int) -> bool:
        """
        Update meta indexed to timestamp

        :param ts: timestamp
        :return: True if updated
        """
        return self.storage.update_meta_indexed_to_ts(ts)

    def clear_meta_indexed_by_id(self, id: int) -> bool:
        """
        Clear meta indexed timestamp by ID

        :param id: ctx ID
        :return: True if cleared
        """
        return self.storage.clear_meta_indexed_by_id(id)

    def clear_meta_indexed_all(self) -> bool:
        """
        Clear all meta indexed timestamps

        :return: True if cleared
        """
        return self.storage.clear_meta_indexed_all()

    def get_groups(self) -> dict:
        """
        Return dict of groups

        :return: dict of ctx meta
        """
        return self.storage.get_groups()

    def insert_group(self, group: CtxGroup):
        """
        Insert group

        :param group: CtxGroup
        """
        return self.storage.insert_group(group)

    def update_group(self, group: CtxGroup):
        """
        Update group

        :param group: CtxGroup
        """
        return self.storage.update_group(group)

    def remove_group(self, id: int, all: bool = False):
        """
        Remove group by ID

        :param id: group ID
        :param all: remove items
        """
        return self.storage.delete_group(id, all=all)

    def truncate_groups(self):
        """Remove groups"""
        return self.storage.truncate_groups()

    def update_meta_group_id(self, id: int, group_id: int):
        """
        Update meta group ID

        :param id: ctx ID
        :param group_id: group ID
        """
        return self.storage.update_meta_group_id(id, group_id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.26 23:00:00                  #
# ================================================== #

from pygpt_net.item.assistant import AssistantItem
from pygpt_net.item.attachment import AttachmentItem
from pygpt_net.provider.core.assistant.json_file import JsonFileProvider

from .files import Files
from .store import Store


class Assistants:
    def __init__(self, window=None):
        """
        Assistants core

        :param window: Window instance
        """
        self.window = window
        self.files = Files(window)
        self.store = Store(window)
        self.provider = JsonFileProvider(window)  # json file provider
        self.current_file = None
        self.items = {}

    def install(self):
        """Install provider data"""
        self.provider.install()
        self.files.install()
        self.store.install()

    def patch(self, app_version) -> bool:
        """
        Patch provider data

        :param app_version: app version
        :return: True if data was patched
        """
        res1 = self.provider.patch(app_version)
        res2 = self.store.patch(app_version)
        return res1 or res2

    def get_by_idx(self, idx: int) -> str:
        """
        Return assistant ID by index

        :param idx: index
        :return: assistant ID
        """
        assistants = self.get_all()
        return list(assistants.keys())[idx]

    def get_by_id(self, id: str) -> AssistantItem | None:
        """
        Return assistant by ID

        :param id: ID
        :return: assistant
        """
        assistants = self.get_all()
        if id not in assistants:
            return None
        return assistants[id]

    def get_all(self) -> dict:
        """
        Return assistants

        :return: assistants dict
        """
        return self.items

    def has(self, id: str) -> bool:
        """
        Check if assistant exists

        :param id: assistant ID
        :return: bool
        """
        return id in self.items

    def create(self) -> AssistantItem:
        """
        Create new assistant item (only empty object)

        :return: assistant ID
        """
        assistant = AssistantItem()
        return assistant

    def add(self, assistant: AssistantItem):
        """
        Add assistant

        :param assistant: item to add
        """
        id = assistant.id
        self.items[id] = assistant  # add assistant

        # save to file
        self.save()

    def delete(self, id: str):
        """
        Delete assistant

        :param id: id of assistant
        """
        if id in self.items:
            self.items.pop(id)
        self.save()

    def clear(self):
        """Clear all assistants"""
        self.items = {}
        self.save()

    def replace_attachment(
            self,
            assistant: AssistantItem,
            attachment: AttachmentItem,
            old_id: str,
            new_id: str
    ):
        """
        Replace temporary attachment with uploaded one

        :param assistant: assistant object
        :param attachment: attachment object
        :param old_id: old id
        :param new_id: new id
        """
        if old_id in assistant.attachments:
            assistant.attachments[new_id] = attachment
            del assistant.attachments[old_id]
            self.save()

    def get_default_assistant(self) -> str | None:
        """
        Return default assistant ID or None

        :return: default assistant
        :rtype: AssistantItem or None
        """
        assistants = self.get_all()
        if len(assistants) == 0:
            return None
        return list(assistants.keys())[0]

    def load(self):
        """Load assistants"""
        self.items = self.provider.load()
        self.store.load()  # load vector stores
        self.files.load()  # load files

    def save(self):
        """Save assistants"""
        self.provider.save(self.items)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.29 07:00:00                  #
# ================================================== #

import os

from PySide6.QtWidgets import QApplication
from PySide6.QtCore import QRect

from pygpt_net.utils import trans


class Layout:
    def __init__(self, window=None):
        """
        Layout controller

        :param window: Window instance
        """
        self.window = window
        # self.splitters = ["main", "main.output", "toolbox", "toolbox.mode", "toolbox.presets"]
        self.splitters = ["main", "main.output", "toolbox", "toolbox.mode", "calendar", "interpreter", "interpreter.columns"]
        self.text_nodes = ["input"]

    def setup(self):
        """Setup layout"""
        self.window.controller.theme.setup()

    def post_setup(self):
        """Post setup layout (after window initialization)"""
        self.text_nodes_restore()
        self.state_restore()
        self.splitters_restore()
        self.tabs_restore()
        self.groups_restore()
        self.scroll_restore()

        # restore plugin settings state
        self.window.controller.plugins.set_by_tab(
            self.window.ui.tabs['plugin.settings'].currentIndex()
        )
        self.window.controller.ui.update_tokens()  # update tokens

    def save(self):
        """Save layout state"""
        self.text_nodes_save()
        self.splitters_save()
        self.tabs_save()
        self.groups_save()
        self.scroll_save()
        self.state_save()

    def text_nodes_restore(self):
        """Restore nodes text"""
        if not self.window.core.config.has('layout.text_nodes'):
            return
        data = self.window.core.config.get('layout.text_nodes')
        for id in self.window.ui.nodes:
            if id in data:
                try:
                    self.window.ui.nodes[id].setText(data[id])
                except Exception as e:
                    print("Error while restoring field state: " + str(e))
                    self.window.core.debug.log(e)

    def text_nodes_save(self):
        """Save nodes text"""
        data = {}
        for id in self.text_nodes:
            data[id] = self.window.ui.nodes[id].toPlainText()
        self.window.core.config.set('layout.text_nodes', data)

    def tabs_save(self):
        """Save tabs state"""
        data = {}
        for tab in self.window.ui.tabs:
            if not isinstance(self.window.ui.tabs[tab], dict):
                data[tab] = self.window.ui.tabs[tab].currentIndex()
        self.window.core.config.set('layout.tabs', data)

    def groups_save(self):
        """Save groups state"""
        data = {}
        for id in self.window.ui.groups:
            data[id] = self.window.ui.groups[id].box.isChecked()
        self.window.core.config.set('layout.groups', data)

    def tabs_restore(self):
        """Restore tabs state"""
        if not self.window.core.config.has('layout.tabs'):
            return
        data = self.window.core.config.get('layout.tabs')
        for tab in self.window.ui.tabs:
            if tab in data:
                try:
                    self.window.ui.tabs[tab].setCurrentIndex(data[tab])
                except Exception as e:
                    print("Error while restoring tab state: " + str(e))
                    self.window.core.debug.log(e)

    def splitters_save(self):
        """Save splitters state"""
        data = {}
        for splitter in self.splitters:
            # do not save main splitter state if notepad was not opened yet
            if splitter == "calendar" and not self.window.controller.notepad.opened_once:
                continue
            data[splitter] = self.window.ui.splitters[splitter].sizes()
        self.window.core.config.set('layout.splitters', data)

    def splitters_restore(self):
        """Restore splitters state"""
        if not self.window.core.config.has('layout.splitters'):
            return
        data = self.window.core.config.get('layout.splitters')
        for splitter in self.splitters:
            if splitter in data:
                try:
                    self.window.ui.splitters[splitter].setSizes(data[splitter])
                except Exception as e:
                    print("Error while restoring splitter state: " + str(e))
                    self.window.core.debug.log(e)

    def groups_restore(self):
        """Restore groups state"""
        if not self.window.core.config.has('layout.groups'):
            return
        data = self.window.core.config.get('layout.groups')
        for id in self.window.ui.groups:
            if id in data:
                try:
                    self.window.ui.groups[id].collapse(data[id])
                except Exception as e:
                    print("Error while restoring group state: " + str(e))
                    self.window.core.debug.log(e)

    def scroll_save(self):
        """Save scroll state"""
        data = {}
        if hasattr(self.window.ui.nodes['output'], 'verticalScrollBar'):
            data['output'] = self.window.ui.nodes['output'].verticalScrollBar().value()

        # notepads
        for id in self.window.ui.notepad:
            scroll_id = "notepad." + str(id)
            data[scroll_id] = self.window.ui.notepad[id].textarea.verticalScrollBar().value()
        self.window.core.config.set('layout.scroll', data)

    def scroll_restore(self):
        """Restore scroll state"""
        if not self.window.core.config.has('layout.scroll'):
            return
        data = self.window.core.config.get('layout.scroll')
        for scroll_id in data:
            # notepads
            if scroll_id.startswith("notepad."):
                id = int(scroll_id.replace("notepad.", ""))
                if id in self.window.ui.notepad:
                    try:
                        self.window.ui.notepad[id].textarea.verticalScrollBar().setValue(data[scroll_id])
                    except Exception as e:
                        print("Error while restoring scroll state: " + str(e))
                        self.window.core.debug.log(e)
        if 'output' in data:
            try:
                if hasattr(self.window.ui.nodes['output'], 'verticalScrollBar'):
                    self.window.ui.nodes['output'].verticalScrollBar().setValue(data['output'])
            except Exception as e:
                print("Error while restoring scroll state: " + str(e))
                self.window.core.debug.log(e)

    def state_restore(self):
        """Restore window state"""
        if not self.window.core.config.has('layout.window'):
            return
        data = self.window.core.config.get('layout.window')
        try:
            screen = QApplication.primaryScreen()
            available_geometry = screen.availableGeometry()

            if 'geometry' in data:
                geometry_data = data['geometry']
                x, y, width, height = (geometry_data['x'],
                                       geometry_data['y'],
                                       geometry_data['width'],
                                       geometry_data['height'])

                window_rect = QRect(x, y, width, height)
                adjusted_rect = available_geometry.intersected(window_rect)

                if not available_geometry.contains(window_rect):
                    adjusted_rect.adjust(0, 0, -20, -20)  # Adjust to fit within the screen

                self.window.move(adjusted_rect.x(), adjusted_rect.y())
                self.window.resize(adjusted_rect.width(), adjusted_rect.height())

            if 'maximized' in data and data['maximized']:
                self.window.showMaximized()
        except Exception as e:
            print("Error while restoring window state: " + str(e))

    def state_save(self):
        """Save window state"""
        data = {}
        geometry_data = {}
        geometry = self.window.geometry()
        geometry_data['x'] = geometry.x()
        geometry_data['y'] = geometry.y()
        geometry_data['width'] = geometry.width()
        geometry_data['height'] = geometry.height()
        data['geometry'] = geometry_data
        data['maximized'] = self.window.isMaximized()
        self.window.core.config.set('layout.window', data)

        # ------------------

        # output zoom save (Chromium, web engine only)
        if hasattr(self.window.ui.nodes['output'], 'get_zoom_value'):
            self.window.core.config.set('zoom', self.window.ui.nodes['output'].get_zoom_value())

    def restore_plugin_settings(self):
        """Restore groups state"""
        if not self.window.core.config.has('layout.groups'):
            return
        data = self.window.core.config.get('layout.groups')
        for id in self.window.ui.groups:
            if id in data and id.startswith('plugin.settings.'):
                try:
                    self.window.ui.groups[id].collapse(data[id])
                except Exception as e:
                    print("Error while restoring group state: " + str(e))
                    self.window.core.debug.log(e)

    def restore_settings(self):
        """Restore groups state"""
        if not self.window.core.config.has('layout.groups'):
            return
        data = self.window.core.config.get('layout.groups')
        for id in self.window.ui.groups:
            if id in data and id.startswith('settings.advanced.'):
                try:
                    self.window.ui.groups[id].collapse(data[id])
                except Exception as e:
                    print("Error while restoring group state: " + str(e))
                    self.window.core.debug.log(e)

    def restore_default_css(self, force: bool = False):
        """
        Restore app CSS to default

        :param force: Force restore
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='restore.css',
                id=0,
                msg=trans('dialog.css.restore.confirm'),
            )
            return

        # restore css
        self.window.core.debug.info("Restoring CSS...")
        self.window.core.filesystem.backup_custom_css()
        self.window.core.filesystem.install_css(force=True)
        self.window.core.debug.info("CSS restored.")

        # update editor if opened
        current_file = self.window.ui.dialog['config.editor'].file
        if current_file is not None:
            if current_file.endswith('.css') and self.window.core.settings.active['editor']:
                self.window.core.settings.load_editor(current_file)

        # show success message
        self.window.ui.dialogs.alert(trans('dialog.css.restore.confirm.success'))
        self.window.core.debug.info("Reloading theme...")
        self.window.controller.theme.reload(force=True)  # reload theme
        self.window.core.debug.info("Theme reloaded.")

    def reload(self):
        """Reload layout"""
        self.post_setup()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.26 23:00:00                  #
# ================================================== #

import json

from pygpt_net.utils import unpack_var
from pygpt_net.item.assistant import AssistantStoreItem


def unpack_store(store: AssistantStoreItem, row: dict) -> AssistantStoreItem:
    """
    Unpack store item from DB row

    :param store: store item (AssistantStoreItem)
    :param row: DB row
    :return: store item
    """
    store.id = row['store_id']
    store.record_id = unpack_var(row['id'], "int")
    store.uuid = row['uuid']
    store.created = unpack_var(row['created_ts'], "int")
    store.updated = unpack_var(row['updated_ts'], "int")
    store.last_active = unpack_var(row['last_active_ts'], "int")
    store.last_sync = unpack_var(row['last_sync_ts'], "int")
    store.name = row['name']
    store.description = row['description']
    store.expire_days = unpack_var(row['expire_days'], "int")
    store.usage_bytes = unpack_var(row['usage_bytes'], "int")
    store.num_files = unpack_var(row['num_files'], "int")
    store.last_status = row['status']
    store.status = unpack_item_value(row['status_json'])
    store.is_thread = unpack_var(row['is_thread'], "bool")
    return store

def pack_item_value(value: any) -> str:
    """
    Pack item value to JSON

    :param value: Value to pack
    :return: JSON string or value itself
    """
    if isinstance(value, (list, dict)):
        return json.dumps(value)
    return value

def unpack_item_value(value: any) -> any:
    """
    Unpack item value from JSON

    :param value: Value to unpack
    :return: Unpacked value
    """
    if value is None:
        return None
    try:
        return json.loads(value)
    except:
        return value


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.22 23:00:00                  #
# ================================================== #

import os
import csv

from PySide6.QtGui import QAction
from pygpt_net.utils import trans


class Template:
    def __init__(self, window=None):
        """
        Prompt templates

        :param window: Window instance
        """
        self.window = window
        self.prompts = {}
        self.loaded = False

    def init(self):
        """Init prompt templates"""
        self.load()

    def load(self):
        """Load prompt templates"""
        if self.loaded:
            return
        try:
            self.load_from_csv()
        except Exception as e:
            self.window.core.debug.log(e)
        self.loaded = True

    def load_from_csv(self):
        """Load prompt templates from CSV"""
        # CSV file is given from: https://github.com/f/awesome-chatgpt-prompts
        path = os.path.join(self.window.core.config.get_app_path(), "data", "prompts.csv")
        self.prompts = {}
        i = 0
        with open(path, newline='', encoding='utf-8') as f:
            data = csv.reader(f, delimiter=',')
            for row in data:
                if i == 0:
                    i += 1
                    continue
                self.prompts[i] = {
                    "name": row[0],
                    "prompt": row[1]
                }
                i += 1
        # sort by name
        self.prompts = dict(sorted(self.prompts.items(), key=lambda item: item[1]['name']))

    def to_menu_options(self, menu, parent: str = "global"):
        """Convert prompts to menu options"""
        self.init()
        menu.addSeparator()
        submenu = menu.addMenu(trans("preset.prompt.paste_template"))
        for key, value in self.prompts.items():
            action = QAction(value['name'], menu)
            action.triggered.connect(lambda checked=False, key=key: self.window.controller.presets.paste_prompt(key, parent))
            action.setToolTip(value['prompt'])
            submenu.addAction(action)

    def get_by_id(self, id: int) -> dict or None:
        """
        Get prompt by id

        :param id: Prompt ID
        """
        self.init()
        return self.prompts.get(id, None)

    def get_all(self) -> dict:
        """
        Get all prompt templates

        :return: Prompt templates
        """
        self.init()
        return self.prompts



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.25 01:00:00                  #
# ================================================== #

import copy
import json
import os
import shutil


class Settings:
    def __init__(self, window=None):
        """
        Settings core

        :param window: Window instance
        """
        self.window = window

        # prepare cfg ids
        self.ids = ['settings', 'editor']
        self.active = {}
        self.options = {}
        self.sections = {}
        self.initialized = False

        # prepare active
        for id in self.ids:
            self.active[id] = False

    def get_options(self, id: str = None) -> dict:
        """
        Return options for given id

        :param id: settings id
        :return: dictionary of options
        """
        if not self.initialized:
            self.load()
        if id is None:
            return self.options
        if id in self.options:
            return self.options[id]

    def get_sections(self, id: str = None) -> dict:
        """
        Return sections for given id

        :param id: settings id
        :return: dictionary of options
        """
        if not self.initialized:
            self.load()
        if id is None:
            return self.sections
        if id in self.sections:
            return self.sections[id]

    def get_persist_options(self) -> list:
        """
        Return persist options keys (options that should be persisted when loading defaults)

        :return: list of keys
        """
        if not self.initialized:
            self.load()
        persist_options = []
        for option in self.options:
            if 'persist' in self.options[option] and self.options[option]['persist']:
                persist_options.append(option)
        return persist_options

    def load(self):
        """Load settings options"""
        self.options = self.window.core.config.get_options()
        self.sections = self.window.core.config.get_sections()
        self.initialized = True

    def load_user_settings(self):
        """Load user config (from user home dir)"""
        # replace config with user base config
        self.window.core.config.load_config()

    def load_app_settings(self):
        """Load base app config (from app root dir)"""
        # persist important and non-settings values
        settings_options = self.get_options()
        persist_options = self.get_persist_options()
        all_options = self.window.core.config.all()
        persist_values = {}
        for option in all_options:
            if option in persist_options or option not in settings_options:
                persist_values[option] = copy.deepcopy(self.window.core.config.get(option))

        # save current config backup
        self.window.core.config.save('config.json.backup')

        # replace config with app base config
        self.window.core.config.from_base_config()

        # restore persisted values
        for option in persist_values:
            self.window.core.config.set(option, persist_values[option])

    def load_default_editor(self):
        """Load defaults from file"""
        file = self.window.ui.dialog['config.editor'].file
        self.load_editor(file)
        self.window.ui.status("Restored from user file: {}".format(file))

    def load_default_editor_app(self):
        """Load defaults from file (app)"""
        file = self.window.ui.dialog['config.editor'].file
        basename = os.path.basename(file)
        if basename.endswith(".css"):
            path = str(os.path.join(self.window.core.config.get_app_path(), "data", "css", basename))
            self.load_editor(file, path)
            self.window.ui.status("Restored from app defaults: {}".format(basename))
        elif basename.endswith(".json"):
            path = str(os.path.join(self.window.core.config.get_app_path(), "data", "config", basename))
            self.load_editor(file, path)
            self.window.ui.status("Restored from app defaults: {}".format(basename))

    def load_editor(self, file: str = None, path: str = None):
        """
        Load file to editor

        :param file: file name (JSON/CSS)
        :param path: file path (force load) or None
        """
        # load file
        if path is None:
            if file.endswith('.json'):
                path = os.path.join(self.window.core.config.get_user_path(), file)
            elif file.endswith('.css'):
                path = os.path.join(self.window.core.config.get_user_path(), 'css', file)
            self.window.ui.paths['config'].setText(path)

        self.window.ui.dialog['config.editor'].file = file
        try:
            with open(path, 'r', encoding="utf-8") as f:
                txt = f.read()
                self.window.ui.editor['config'].setPlainText(txt)
        except Exception as e:
            self.window.core.debug.log(e)
            self.window.ui.status("Error loading file: {}".format(e))

    def save_editor(self):
        """Save file to disk"""
        file = self.window.ui.dialog['config.editor'].file
        path = None
        data = self.window.ui.editor['config'].toPlainText()

        # check if this is a valid JSON
        if file.endswith('.json'):
            try:
                json.loads(data)
            except Exception as e:
                self.window.ui.status("This is not a valid JSON: {}".format(e))
                self.window.ui.dialogs.alert("This is not a valid JSON: {}".format(e))
                return
            path = os.path.join(self.window.core.config.get_user_path(), file)
        elif file.endswith('.css'):
            path = os.path.join(self.window.core.config.get_user_path(), 'css', file)

        if path is None:
            self.window.ui.status("Error saving file: invalid file name")
            return

        # make backup of current file
        backup_file = file + '.backup'
        if file.endswith('.css'):
            backup_path = os.path.join(self.window.core.config.get_user_path(), "css", backup_file)
        else:
            backup_path = os.path.join(self.window.core.config.get_user_path(), backup_file)
        if os.path.isfile(path):
            shutil.copyfile(path, backup_path)
            self.window.ui.status("Created backup file: {}".format(backup_file))

        # save changes to current file
        try:
            with open(path, 'w', encoding="utf-8") as f:
                f.write(data)
            self.window.ui.status("Saved file: {}".format(path))
            self.window.ui.dialogs.alert("Saved file: {}".format(path))
            if file == "config.json":
                self.window.core.config.load_config()  # reload config
            elif file == "models.json":
                self.window.core.models.load()  # reload models
            elif file.endswith('.css'):
                self.window.controller.chat.render.on_theme_change()
                self.window.controller.theme.reload(force=True)  # reload theme
        except Exception as e:
            self.window.core.debug.log(e)
            self.window.ui.status("Error saving file: {}".format(path))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.17 01:00:00                  #
# ================================================== #

import json

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLabel, QVBoxLayout, QWidget, QCheckBox, QHBoxLayout, QScrollArea, \
    QSizePolicy

from pygpt_net.ui.widget.element.labels import HelpLabel, UrlLabel
from pygpt_net.ui.widget.option.combo import OptionCombo
from pygpt_net.ui.widget.option.input import OptionInput
from pygpt_net.utils import trans


class WebTab:
    def __init__(self, window=None):
        """
        Tab: Web

        :param window: Window instance
        """
        self.window = window
        self.params_scroll = None
        self.params_widget = None

    def setup(self):
        """
        Setup tab widget
        """
        # get loaders list
        loaders = self.window.controller.config.placeholder.apply_by_id("llama_index_loaders_web")
        loaders_list = []
        for loader in loaders:
            key = list(loader.keys())[0]
            loaders_list.append(key.replace("web_", ""))

        self.window.ui.nodes["tool.indexer.web.loader"] = OptionCombo(
            self.window,
            "tool.indexer",
            "web.loader",
            {
                "label": trans("tool.indexer.tab.web.loader"),
                "keys": loaders_list,
                "value": "webpage",
            }
        )

        self.window.ui.nodes["tool.indexer.web.loader"].layout.setContentsMargins(0, 0, 0, 0)
        self.window.ui.nodes["tool.indexer.web.loader.label"] = QLabel(trans("tool.indexer.tab.web.loader"))
        self.window.ui.add_hook("update.tool.indexer.web.loader", self.hook_loader_change)

        self.window.ui.nodes["tool.indexer.web.options.label"] = HelpLabel(trans("tool.indexer.tab.web.source"))
        self.window.ui.nodes["tool.indexer.web.config.label"] = HelpLabel(trans("tool.indexer.tab.web.cfg"))
        self.window.ui.nodes["tool.indexer.web.config.help"] = UrlLabel(
            trans("tool.indexer.tab.web.help"),
            "https://pygpt.readthedocs.io/en/latest/modes.html#chat-with-files-llama-index")

        # --------------------------------------------------

        params_layout = QVBoxLayout()
        params_layout.setContentsMargins(0, 0, 0, 0)

        self.params_scroll = QScrollArea()
        self.params_scroll.setWidgetResizable(True)
        self.window.ui.nodes["tool.indexer.web.loader.option"] = {}
        self.window.ui.nodes["tool.indexer.web.loader.option_group"] = {}
        self.window.ui.nodes["tool.indexer.web.loader.config"] = {}
        self.window.ui.nodes["tool.indexer.web.loader.config_group"] = {}

        # params
        params_layout.addWidget(self.window.ui.nodes["tool.indexer.web.options.label"])
        inputs, groups = self.setup_loader_options()
        for loader in inputs:
            for k in inputs[loader]:
                self.window.ui.nodes["tool.indexer.web.loader.option." + loader + "." + k] = inputs[loader][k]
        for loader in groups:
            self.window.ui.nodes["tool.indexer.web.loader.option_group"][loader] = groups[loader]
            params_layout.addWidget(self.window.ui.nodes["tool.indexer.web.loader.option_group"][loader])
            self.window.ui.nodes["tool.indexer.web.loader.option_group"][loader].hide()  # hide on start

        # config
        params_layout.addWidget(self.window.ui.nodes["tool.indexer.web.config.label"])
        inputs, groups = self.setup_loader_config()
        for loader in inputs:
            for k in inputs[loader]:
                self.window.ui.nodes["tool.indexer.web.loader.config." + loader + "." + k] = inputs[loader][k]
        for loader in groups:
            self.window.ui.nodes["tool.indexer.web.loader.config_group"][loader] = groups[loader]
            params_layout.addWidget(self.window.ui.nodes["tool.indexer.web.loader.config_group"][loader])
            self.window.ui.nodes["tool.indexer.web.loader.config_group"][loader].hide()  # hide on start
        params_layout.addWidget(self.window.ui.nodes["tool.indexer.web.config.help"], alignment=Qt.AlignCenter)

        params_layout.addStretch(1)

        self.params_widget = QWidget()
        self.params_widget.setLayout(params_layout)
        self.params_scroll.setWidget(self.params_widget)

        # resize
        self.params_scroll.setWidgetResizable(True)
        self.params_scroll.setSizePolicy(QSizePolicy.Preferred, QSizePolicy.Expanding)

        # --------------------------------------------------

        self.window.ui.nodes["tool.indexer.web.options.replace"] = QCheckBox(
            trans("tool.indexer.option.replace"))
        self.window.ui.nodes["tool.indexer.web.options.replace"].setChecked(True)

        self.window.ui.nodes["tool.indexer.provider"] = HelpLabel(self.window.core.config.get("llama.idx.storage"))

        loader_layout = QHBoxLayout()
        loader_layout.addWidget(self.window.ui.nodes["tool.indexer.web.loader.label"])
        loader_layout.addWidget(self.window.ui.nodes["tool.indexer.web.loader"])

        options_layout = QVBoxLayout()
        options_layout.addWidget(self.window.ui.nodes["tool.indexer.web.options.replace"])

        self.window.ui.nodes["tool.indexer.web.header.tip"] = HelpLabel(trans("tool.indexer.tab.web.tip"))

        # layout
        layout = QVBoxLayout()
        layout.addWidget(self.window.ui.nodes["tool.indexer.web.header.tip"])
        layout.addLayout(loader_layout)
        layout.addLayout(options_layout)
        layout.addWidget(self.params_scroll)

        # defaults
        self.window.ui.nodes["tool.indexer.web.loader"].set_value("webpage")

        widget = QWidget()
        widget.setLayout(layout)

        return widget

    def hook_loader_change(self, key, value, caller, *args, **kwargs):
        """
        Hook: on loader change

        :param key: Option key
        :param value: Option value
        :param caller: Caller
        :param args: Args
        :param kwargs: Kwargs
        """
        # hide/show options
        for loader in self.window.ui.nodes["tool.indexer.web.loader.option_group"]:
            self.window.ui.nodes["tool.indexer.web.loader.option_group"][loader].hide()
        if value in self.window.ui.nodes["tool.indexer.web.loader.option_group"]:
            self.window.ui.nodes["tool.indexer.web.loader.option_group"][value].show()

        # show/hide label if options are available
        self.window.ui.nodes["tool.indexer.web.options.label"].hide()
        if value in self.window.ui.nodes["tool.indexer.web.loader.option_group"]:
            self.window.ui.nodes["tool.indexer.web.options.label"].show()

        # hide/show config
        for loader in self.window.ui.nodes["tool.indexer.web.loader.config_group"]:
            self.window.ui.nodes["tool.indexer.web.loader.config_group"][loader].hide()
        if value in self.window.ui.nodes["tool.indexer.web.loader.config_group"]:
            self.window.ui.nodes["tool.indexer.web.loader.config_group"][value].show()

        # show/hide label if config are available
        self.window.ui.nodes["tool.indexer.web.config.label"].hide()
        self.window.ui.nodes["tool.indexer.web.config.help"].hide()
        if value in self.window.ui.nodes["tool.indexer.web.loader.config_group"]:
            self.window.ui.nodes["tool.indexer.web.config.label"].show()
            self.window.ui.nodes["tool.indexer.web.config.help"].show()

        self.params_widget.adjustSize()
        self.params_scroll.update()

    def setup_loader_options(self):
        """
        Setup loader options
        """
        inputs = {}
        groups = {}
        loaders = self.window.core.idx.indexing.get_external_instructions()
        for loader in loaders:
            params = loaders[loader]
            inputs[loader] = {}
            group = QVBoxLayout()
            for k in params["args"]:
                widget = OptionInput(self.window, "tool.indexer", f"web.loader.{loader}.option.{k}", {
                    "label": k,
                    "value": "",
                })
                widget.setPlaceholderText(params["args"][k]["type"])
                inputs[loader][k] = widget
                row = QHBoxLayout()  # cols
                row.addWidget(QLabel(k))
                row.addWidget(widget)
                group.addLayout(row)
            group_widget = QWidget()
            group_widget.setLayout(group)
            groups[loader] = group_widget

        return inputs, groups

    def setup_loader_config(self):
        """
        Setup loader config
        """
        inputs = {}
        groups = {}
        loaders = self.window.core.idx.indexing.get_external_config()
        for loader in loaders:
            params = loaders[loader]
            inputs[loader] = {}
            group = QVBoxLayout()
            for k in params:
                widget = OptionInput(self.window, "tool.indexer", f"web.loader.{loader}.config.{k}", {
                    "label": k,
                    "value": params[k]["value"],
                })
                try:
                    if params[k]["value"] is not None:
                        if params[k]["type"] == "list" and isinstance(params[k]["value"], list):
                            widget.setText(", ".join(params[k]["value"]))
                        elif params[k]["type"] == "dict" and isinstance(params[k]["dict"], dict):
                            widget.setText(json.dumps(params[k]["value"]))
                        else:
                            widget.setText(str(params[k]["value"]))
                except Exception as e:
                    self.window.core.debug.log(e)

                widget.setPlaceholderText(params[k]["type"])
                inputs[loader][k] = widget
                row = QHBoxLayout()  # cols
                row.addWidget(QLabel(k))
                row.addWidget(widget)
                group.addLayout(row)
            group_widget = QWidget()
            group_widget.setLayout(group)
            groups[loader] = group_widget

        return inputs, groups



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.14 18:00:00                  #
# ================================================== #

import hashlib
import os
import shutil

from PySide6 import QtGui, QtCore
from PySide6.QtGui import QAction, QIcon
from PySide6.QtWidgets import QFileDialog

from pygpt_net.tools.base import BaseTool
from pygpt_net.tools.image_viewer.ui.dialogs import DialogSpawner
from pygpt_net.utils import trans


class ImageViewer(BaseTool):
    def __init__(self, *args, **kwargs):
        """
        Image viewer

        :param window: Window instance
        """
        super(ImageViewer, self).__init__(*args, **kwargs)
        self.id = "viewer"
        self.width = 640
        self.height = 400
        self.instance_id = 0
        self.spawner = None

    def setup(self):
        """Setup tool"""
        self.spawner = DialogSpawner(self.window)

    def prepare_id(self, file: str):
        """
        Prepare unique id for file

        :param file: file name
        :return: unique id
        """
        return 'image_viewer_' + hashlib.md5(file.encode('utf-8')).hexdigest()

    def new(self):
        """New image viewer dialog"""
        self.open_preview()

    def open_file(self, id: str, auto_close: bool = True,):
        """
        Open image file dialog

        :param id: dialog id
        :param auto_close: auto close current dialog
        """
        path, _ = QFileDialog.getOpenFileName(
            self.window,
            trans("action.open"),
            "",
            "Image files (*.png *.jpg *.jpeg *.bmp *.gif *.tiff *webp);; All files (*.*)")
        if path:
            self.open_preview(path, id, auto_close)

    def open_preview(
            self,
            path: str = None,
            current_id: str = None,
            auto_close: bool = True):
        """
        Open image preview in dialog

        :param path: path to image
        :param current_id: current dialog id
        :param auto_close: auto close current dialog
        """
        if path:
            id = self.prepare_id(path)
            if current_id and auto_close:
                if id != current_id:
                    self.close_preview(current_id)
        else:
            # new instance id
            id = 'image_viewer_' + str(self.instance_id)
            self.instance_id += 1

        self.window.ui.dialogs.open_instance(
            id,
            width=self.width,
            height=self.height,
            type="image_viewer",
        )

        w = self.width
        h = self.height
        img_suffix = ""

        if path is None:
            path = self.window.ui.dialog[id].source.path  # previous img

        if path is None:
            pixmap = QtGui.QPixmap(0, 0)  # blank image
            self.window.ui.dialog[id].source.setPixmap(pixmap)
            self.window.ui.dialog[id].pixmap.path = None
            self.window.ui.dialog[id].pixmap.resize(w, h)
        else:
            pixmap = QtGui.QPixmap(path)
            img_suffix = " ({}x{}px)".format(pixmap.width(), pixmap.height())
            file_size = self.window.core.filesystem.sizeof_fmt(os.path.getsize(path))
            img_suffix += " - {}".format(file_size)
            self.window.ui.dialog[id].source.setPixmap(pixmap)
            self.window.ui.dialog[id].pixmap.path = path
            self.window.ui.dialog[id].pixmap.resize(w, h)

        if path is not None:
            self.window.ui.dialog[id].path = path
            self.window.ui.dialog[id].setWindowTitle(os.path.basename(path) + img_suffix)
        else:
            self.window.ui.dialog[id].path = None
            self.window.ui.dialog[id].setWindowTitle("Image Viewer")

        self.window.ui.dialog[id].resize(w - 1, h - 1)  # redraw fix
        self.window.ui.dialog[id].resize(w, h)
        self.window.ui.dialog[id].show()

    def close_preview(self, id: str):
        """
        Close image preview dialog

        :param id: dialog id
        """
        self.window.ui.dialog[id].resize(self.width - 1, self.height - 1)  # redraw fix
        self.window.ui.dialog[id].close()

    def open_images(self, paths: list):
        """
        Open image in dialog

        :param paths: paths to images
        """
        num_images = len(paths)
        resize_to = 512
        if num_images > 1:
            resize_to = 256
        w = 520
        h = 520
        i = 0
        for path in paths:
            pixmap = QtGui.QPixmap(path)
            pixmap = pixmap.scaled(resize_to, resize_to, QtCore.Qt.KeepAspectRatio)
            self.window.ui.nodes['dialog.image.pixmap'][i].path = path
            self.window.ui.nodes['dialog.image.pixmap'][i].setPixmap(pixmap)
            self.window.ui.nodes['dialog.image.pixmap'][i].setVisible(True)
            i += 1

        # hide unused images
        for j in range(i, 4):
            self.window.ui.nodes['dialog.image.pixmap'][j].setVisible(False)

        # resize dialog
        self.window.ui.dialog['image'].resize(w, h)
        self.window.ui.dialog['image'].show()

    def close_images(self):
        """Close image dialog"""
        self.window.ui.dialog['image'].close()

    def open(self, path: str):
        """
        Open image in default image viewer

        :param path: path to image
        """
        if os.path.exists(path):
            self.window.controller.files.open(path)

    def open_dir(self, path: str):
        """
        Open image in default image viewer

        :param path: path to image
        """
        if os.path.exists(path):
            self.window.controller.files.open_dir(
                path,
                True,
            )
    def save_by_id(self, id: str):
        """
        Save image by dialog id

        :param id: dialog id
        """
        if id in self.window.ui.dialog:
            path = self.window.ui.dialog[id].path
            if path:
                self.save(path)
            else:
                self.window.ui.status("No image to save")

    def save(self, path: str):
        """
        Save image

        :param path: path to image
        """
        if path is None:
            self.window.ui.status("No image to save")
            return

        save_path = QFileDialog.getSaveFileName(
            self.window,
            trans('img.save.title'),
            os.path.basename(path),
            "Image files (*.png *.jpg *.jpeg *.bmp *.gif *.tiff *webp);; All files (*.*)"
        )
        if save_path:
            try:
                if save_path[0] == '':
                    return
                shutil.copyfile(path, save_path[0])
                self.window.ui.status(trans('status.img.saved') + ": {}".format(os.path.basename(save_path[0])))
            except Exception as e:
                self.window.core.debug.log(e)

    def delete(self, path: str, force: bool = False):
        """
        Delete image

        :param path: path to image
        :param force: force delete without confirmation
        """
        if path is None:
            self.window.ui.status("No image to delete")
            return

        if not force:
            self.window.ui.dialogs.confirm(
                type='img_delete',
                id=path,
                msg=trans('confirm.img.delete'),
            )
            return
        try:
            os.remove(path)
            for i in range(0, 4):
                if self.window.ui.nodes['dialog.image.pixmap'][i].path == path:
                    self.window.ui.nodes['dialog.image.pixmap'][i].setVisible(False)
        except Exception as e:
            self.window.core.debug.log(e)

    def setup_menu(self) -> dict:
        """
        Setup main menu

        :return dict with menu actions
        """
        actions = {}
        actions["image.viewer"] = QAction(
            QIcon(":/icons/image.svg"),
            trans("menu.tools.image.viewer"),
            self.window,
            checkable=False,
        )
        actions["image.viewer"].triggered.connect(
            lambda: self.open_preview()
        )
        return actions

    def get_instance(self, type_id: str, dialog_id: str = None):
        """
        Spawn and return dialog instance

        :param type_id: dialog instance type ID
        :param dialog_id: dialog instance ID
        :return dialog instance
        """
        if type_id == "image_viewer":
            return self.spawner.setup(dialog_id)

    def get_lang_mappings(self) -> dict:
        """
        Get language mappings

        :return: dict with language mappings
        """
        return {
            'menu.text': {
                'tools.image.viewer': 'menu.tools.image.viewer',
            }
        }


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.01.19 05:00:00                  #
# ================================================== #

import datetime
import os

from pygpt_net.item.ctx import CtxItem
from pygpt_net.provider.core.history.base import BaseProvider
from .patch import Patch


class TxtFileProvider(BaseProvider):
    def __init__(self, window=None):
        super(TxtFileProvider, self).__init__(window)
        self.window = window
        self.patcher = Patch(window)
        self.id = "txt_file"
        self.type = "history"

    def install(self):
        """
        Install provider data
        """
        history_dir = self.window.core.config.get_user_dir('history')
        if not os.path.exists(history_dir):
            os.mkdir(history_dir)

    def append(self, ctx: CtxItem, mode: str):
        """
        Append text to file

        :param ctx: CtxItem instance
        :param mode: mode (input | output)
        """
        text = ""
        ts = 0
        if mode == "input":
            text = ctx.input
            ts = ctx.input_timestamp
        elif mode == "output":
            text = ctx.output
            ts = ctx.output_timestamp

        # check if text is empty
        if text is None or text.strip() == "":
            return

        path = self.window.core.config.get_user_dir('history')
        name = datetime.datetime.fromtimestamp(ts).strftime("%Y_%m_%d") + ".txt"

        # check directory
        if not os.path.exists(path):
            try:
                os.makedirs(path)
            except Exception as e:
                self.window.core.debug.log(e)
                print("Error creating history directory: " + str(e))

        # append to file
        if os.path.exists(path):
            f = os.path.join(path, name)
            try:
                with open(f, 'a', encoding="utf-8") as file:
                    prefix = ""
                    if self.window.core.config.get('store_history_time'):
                        prefix = datetime.datetime.fromtimestamp(ts).strftime("%H:%M:%S") + ": "
                    file.write(prefix + text + "\n")
            except Exception as e:
                self.window.core.debug.log(e)
                print("Error appending to history: " + str(e))

    def truncate(self):
        """Delete all"""
        path = self.window.core.config.get_user_dir('history')
        try:
            if not os.path.exists(path):
                return
            for f in os.listdir(path):
                os.remove(os.path.join(path, f))
        except Exception as e:
            self.window.core.debug.log(e)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.17 01:00:00                  #
# ================================================== #

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QDialog, QDialogButtonBox, QVBoxLayout, QPlainTextEdit

from pygpt_net.utils import trans


class AlertDialog(QDialog):
    def __init__(self, window=None):
        """
        Alert dialog

        :param window: main window
        """
        super(AlertDialog, self).__init__(window)
        self.window = window
        self.setWindowTitle(trans('alert.title'))
        self.setWindowFlags(self.windowFlags() | Qt.WindowStaysOnTopHint)  # always on top

        QBtn = QDialogButtonBox.Ok

        self.buttonBox = QDialogButtonBox(QBtn)
        self.buttonBox.accepted.connect(self.accept)

        self.layout = QVBoxLayout()
        self.message = QPlainTextEdit()
        self.message.setReadOnly(True)
        self.message.setMaximumWidth(400)
        self.layout.addWidget(self.message)
        self.layout.addWidget(self.buttonBox)
        self.setLayout(self.layout)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.30 04:00:00                  #
# ================================================== #


from PySide6.QtWidgets import QApplication, QFileDialog

from pygpt_net.utils import trans


class Batch:
    def __init__(self, window=None):
        """
        Assistants batch controller

        :param window: Window instance
        """
        self.window = window
        self.files_to_upload = []

    def import_assistants(self, force: bool = False):
        """
        Import all remote assistants from API

        :param force: if true, imports without confirmation
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.import',
                id='',
                msg=trans('confirm.assistant.import'),
            )
            return

        # run asynchronous
        self.window.ui.status("Importing assistants...please wait...")
        self.window.core.gpt.assistants.importer.import_assistants()

    def import_stores(self, force: bool = False):
        """
        Import vector stores from API

        :param force: if true, imports without confirmation
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.store.import',
                id='',
                msg=trans('confirm.assistant.store.import'),
            )
            return
        # run asynchronous
        self.window.ui.status("Importing vector stores...please wait...")
        self.window.core.assistants.store.truncate()  # clear all stores
        self.window.core.gpt.assistants.importer.import_vector_stores()
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.update()

    def import_files_current(self):
        """Import files from API"""
        id = self.window.core.config.get('assistant')
        if id is None or id == "":
            return
        if self.window.core.assistants.has(id):
            assistant = self.window.core.assistants.get_by_id(id)
            store_id = assistant.vector_store
            if store_id is None or store_id == "":
                self.window.ui.dialogs.alert(trans("dialog.assistant.store.alert.assign"))
                return
            self.import_store_files(store_id)

    def import_files(self, force: bool = False):
        """
        Sync files with API (all)

        :param force: force sync files
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.files.import.all',
                id='',
                msg=trans('confirm.assistant.import_files'),
            )
            return
        try:
            self.window.controller.assistant.files.import_files()  # all
        except Exception as e:
            self.window.core.debug.log(e)
            self.window.ui.dialogs.alert(e)

    def import_store_files(self, store_id: str, force: bool = False):
        """
        Sync files with API (store)

        :param store_id: vector store ID
        :param force: force sync files
        """
        if store_id is None:
            self.window.ui.dialogs.alert(trans("dialog.assistant.store.alert.select"))
            return

        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.files.import.store',
                id=store_id,
                msg=trans('confirm.assistant.import_files.store'),
            )
            return
        try:
            self.window.controller.assistant.files.import_files(store_id)  # by store
        except Exception as e:
            self.window.core.debug.log(e)
            self.window.ui.dialogs.alert(e)

    def truncate_files(self, force: bool = False):
        """
        Truncate all files in API

        :param force: if true, imports without confirmation
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.files.truncate',
                id='',
                msg=trans('confirm.assistant.files.truncate'),
            )
            return
        # run asynchronous
        self.window.ui.status("Removing files...please wait...")
        QApplication.processEvents()
        self.window.core.gpt.assistants.importer.truncate_files()  # remove all files from API

    def truncate_store_files_by_idx(self, idx: int, force: bool = False):
        """
        Truncate all files in API (store)

        :param idx: store index
        :param force: if true, imports without confirmation
        """
        store_id = self.window.controller.assistant.store.get_by_tab_idx(idx)
        self.truncate_store_files(store_id, force)

    def truncate_store_files(self, store_id: str, force: bool = False):
        """
        Truncate all files in API (store)

        :param store_id: store ID
        :param force: if true, imports without confirmation
        """
        if store_id is None:
            self.window.ui.dialogs.alert(trans("dialog.assistant.store.alert.select"))
            return

        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.files.truncate.store',
                id=store_id,
                msg=trans('confirm.assistant.files.truncate.store'),
            )
            return
        # run asynchronous
        self.window.ui.status("Removing files...please wait...")
        QApplication.processEvents()
        self.window.core.gpt.assistants.importer.truncate_files(store_id)  # remove all files from API

    def clear_store_files_by_idx(self, idx: int, force: bool = False):
        """
        Clear files (store, local only)

        :param idx: store index
        :param force: if true, clears without confirmation
        """
        store_id = self.window.controller.assistant.store.get_by_tab_idx(idx)
        self.clear_store_files(store_id, force)

    def clear_store_files(self, store_id: str = None, force: bool = False):
        """
        Clear files (store, local only)

        :param store_id: store ID
        :param force: if true, clears without confirmation
        """
        if store_id is None:
            self.window.ui.dialogs.alert(trans("dialog.assistant.store.alert.select"))
            return

        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.files.clear.store',
                id=store_id,
                msg=trans('confirm.assistant.files.clear'),
            )
            return
        self.window.ui.status("Clearing store files...please wait...")
        QApplication.processEvents()
        self.window.core.assistants.files.truncate_local(store_id)  # clear files local
        self.window.controller.assistant.files.update()
        self.window.ui.status("OK. All store files cleared.")
        self.window.ui.dialogs.alert(trans("status.finished"))

    def clear_files(self, force: bool = False):
        """
        Clear files (all, local only)

        :param force: if true, clears without confirmation
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.files.clear.all',
                id='',
                msg=trans('confirm.assistant.files.clear'),
            )
            return
        self.window.ui.status("Clearing files...please wait...")
        QApplication.processEvents()
        self.window.core.assistants.files.truncate_local()  # clear files local
        self.window.controller.assistant.files.update()
        self.window.ui.status("OK. All files cleared.")
        self.window.ui.dialogs.alert(trans("status.finished"))

    def clear_stores(self, force: bool = False):
        """
        Clear vector stores (local only)

        :param force: if true, clears without confirmation
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.store.clear',
                id='',
                msg=trans('confirm.assistant.store.clear'),
            )
            return
        self.window.ui.status("Clearing vector stores...please wait...")
        QApplication.processEvents()
        self.window.core.assistants.store.truncate()  # clear all stores
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.update()
        self.window.ui.status("OK. All stores cleared.")
        self.window.controller.assistant.store.current = None
        self.window.controller.assistant.store.init()

    def truncate_stores(self, force: bool = False):
        """
        Truncate vector stores in API

        :param force: if true, truncates without confirmation
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.store.truncate',
                id='',
                msg=trans('confirm.assistant.store.truncate'),
            )
            return
        # run asynchronous
        self.window.ui.status("Removing vector stores...please wait...")
        QApplication.processEvents()
        self.window.core.assistants.store.truncate()  # clear all stores
        self.window.core.gpt.assistants.importer.truncate_vector_stores()
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.update()
        self.window.controller.assistant.store.current = None
        self.window.controller.assistant.store.init()

    def refresh_stores(self, force: bool = False):
        """
        Refresh all vector stores

        :param force: if true, refresh without confirmation
        """
        if not force:
            self.window.ui.dialogs.confirm(
                type='assistant.store.refresh',
                id='',
                msg=trans('confirm.assistant.store.refresh'),
            )
            return
        self.window.ui.status("Refreshing vector stores...please wait...")
        QApplication.processEvents()
        self.window.core.gpt.assistants.importer.refresh_vector_stores()

    def handle_imported_assistants(self, num: int):
        """
        Handle imported assistants

        :param num: number of imported assistants
        """
        self.window.controller.assistant.update()
        self.window.controller.assistant.store.update()
        self.window.controller.assistant.files.update()
        self.window.ui.status("OK. Imported assistants: " + str(num) + ".")
        self.window.ui.dialogs.alert(trans("status.finished"))

    def handle_imported_assistants_failed(self, error: any):
        """
        Handle error on importing assistants

        :param error: error message
        """
        self.window.core.debug.log(error)
        print("Error importing assistants")
        self.window.controller.assistant.update()
        self.window.ui.status("Error importing assistants.")
        self.window.ui.dialogs.alert(error)

    def handle_refreshed_stores(self, num: int):
        """
        Handle refreshed stores

        :param num: number of refreshed files
        """
        self.window.controller.assistant.store.update_current()
        self.window.controller.assistant.store.update()
        self.window.ui.dialogs.alert(trans("status.finished"))
        self.window.ui.status("OK. All stores refreshed.")

    def handle_refreshed_stores_failed(self, error: any):
        """
        Handle error on refreshing stores

        :param error: error message
        """
        self.window.core.debug.log(error)
        print("Error refreshing stores", error)
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.update()
        self.window.ui.status("Error refreshing stores.")
        self.window.ui.dialogs.alert(error)

    def handle_imported_stores(self, num: int):
        """
        Handle imported stores

        :param num: number of imported files
        """
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.update()
        self.window.ui.status("OK. Imported stores: " + str(num) + ".")
        self.window.ui.dialogs.alert(trans("status.finished"))

    def handle_imported_stores_failed(self, error: any):
        """
        Handle error on importing stores

        :param error: error message
        """
        self.window.core.debug.log(error)
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.update()
        print("Error importing stores", error)
        self.window.ui.status("Error importing stores.")
        self.window.ui.dialogs.alert(error)

    def handle_truncated_stores(self, num: int):
        """
        Handle truncated stores

        :param num: number of removed files
        """
        self.window.core.assistants.store.truncate()
        self.remove_all_stores_from_assistants()
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.update()
        self.window.controller.assistant.store.current = None
        self.window.controller.assistant.store.init()
        self.window.ui.status("OK. Removed stores: " + str(num) + ".")
        self.window.ui.dialogs.alert(trans("status.finished"))

    def handle_truncated_stores_failed(self, error: any):
        """
        Handle error on truncating stores

        :param error: error message
        """
        self.window.core.debug.log(error)
        print("Error removing stores", error)
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.update()
        self.window.ui.status("Error removing stores.")
        self.window.ui.dialogs.alert(error)

    def handle_imported_files(self, num: int):
        """
        Handle imported files

        :param num: number of imported files
        """
        self.window.controller.assistant.files.update()
        self.window.ui.status("OK. Imported files: " + str(num) + ".")
        self.window.ui.dialogs.alert(trans("status.finished"))

    def handle_imported_files_failed(self, error: any):
        """
        Handle error on importing files

        :param error: error message
        """
        self.window.core.debug.log(error)
        self.window.controller.assistant.files.update()
        print("Error importing files")
        self.window.ui.status("Error importing files.")
        self.window.ui.dialogs.alert(error)

    def handle_truncated_files(self, store_id: str = None, num: int = 0):
        """
        Handle truncated (in API) files

        :param store_id: vector store ID
        :param num: number of truncated files
        """
        self.window.ui.status("OK. Truncated files: " + str(num) + ".")
        if store_id is not None:
            self.window.controller.assistant.store.refresh_by_store_id(store_id)
        self.window.controller.assistant.files.update()
        self.window.ui.dialogs.alert(trans("status.finished"))

    def handle_truncated_files_failed(self, error: any):
        """
        Handle error on truncated files

        :param error: error message
        """
        self.window.core.debug.log(error)
        print("Error truncating files")
        self.window.controller.assistant.files.update()
        self.window.ui.status("Error truncating files.")
        self.window.ui.dialogs.alert(error)

    def remove_store_from_assistants(self, store_id: str):
        """
        Remove vector store from all assistants after store deletion

        :param store_id: vector store ID
        """
        for id in list(self.window.core.assistants.items.keys()):
            assistant = self.window.core.assistants.get_by_id(id)
            if assistant is not None:
                if assistant.vector_store == store_id:
                    assistant.vector_store = None  # remove from assistant

        self.window.core.assistants.save()
        self.window.core.assistants.files.on_store_deleted(store_id)  # remove from files

    def remove_all_stores_from_assistants(self):
        """Remove all vector stores from all assistants"""
        for id in list(self.window.core.assistants.items.keys()):
            assistant = self.window.core.assistants.get_by_id(id)
            if assistant is not None:
                assistant.vector_store = None

        self.window.core.assistants.save()
        self.window.core.assistants.files.on_all_stores_deleted()  # remove all from files

    def open_upload_files(self):
        """Open upload files dialog"""
        if self.window.controller.assistant.store.current is None:
            self.window.ui.dialogs.alert("Please select vector store first.")
            return

        options = QFileDialog.Options()
        files, _ = QFileDialog.getOpenFileNames(
            self.window,
            "Select file(s)...",
            "",
            "All Files (*)",
            options=options
        )
        if files:
            self.files_to_upload = files

        if self.files_to_upload:
            msg = "Are you sure you want to upload {} file(s)?".format(len(self.files_to_upload))
            self.window.ui.dialogs.confirm(
                type="assistant.files.upload",
                id=0,
                msg=msg,
            )

    def open_upload_dir(self):
        """Open upload files dialog"""
        if self.window.controller.assistant.store.current is None:
            self.window.ui.dialogs.alert("Please select vector store first.")
            return

        options = QFileDialog.Options()
        directory = QFileDialog.getExistingDirectory(
            self.window,
            "Select directory...",
            options=options
        )
        if directory:
            self.files_to_upload = self.window.core.filesystem.get_files_from_dir(directory)

        if self.files_to_upload:
            msg = ("Are you sure you want to upload {} file(s) from directory {}?".
                   format(len(self.files_to_upload), directory))
            self.window.ui.dialogs.confirm(
                type="assistant.files.upload",
                id=0,
                msg=msg,
            )

    def upload(self, force: bool = False):
        """
        Upload files to vector store

        :param force: if true, uploads without confirmation
        """
        if self.window.controller.assistant.store.current is None:
            self.window.ui.dialogs.alert(trans("dialog.assistant.store.alert.select"))
            return

        store_id = self.window.controller.assistant.store.current
        self.window.ui.status("Uploading files...please wait...")
        QApplication.processEvents()
        self.window.core.gpt.assistants.importer.upload_files(store_id, self.files_to_upload)
        self.files_to_upload = []  # clear files

    def handle_uploaded_files(self, num: int):
        """
        Handle uploaded files

        :param num: number of uploaded files
        """
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.refresh_status()
        self.window.ui.status("OK. Uploaded files: " + str(num) + ".")
        self.window.ui.dialogs.alert("OK. Uploaded files: " + str(num) + ".")

    def handle_uploaded_files_failed(self, error: any):
        """
        Handle error on uploading files

        :param error: error message
        """
        self.window.core.debug.log(error)
        print("Error uploading files")
        self.window.controller.assistant.files.update()
        self.window.controller.assistant.store.refresh_status()
        self.window.ui.status("Error uploading files.")
        self.window.ui.dialogs.alert(error)

    def handle_status_change(self, mode: str, msg: str):
        """
        Handle status change

        :param mode: status mode
        :param msg: status message
        """
        self.window.ui.status(msg)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.03.26 15:00:00                  #
# ================================================== #

from PySide6.QtWidgets import QPushButton, QHBoxLayout, QVBoxLayout

from pygpt_net.ui.widget.dialog.base import BaseDialog
from pygpt_net.ui.widget.dialog.editor_file import EditorFileDialog
from pygpt_net.ui.widget.element.labels import HelpLabel
from pygpt_net.utils import trans

from .widgets import TextFileEditor

class DialogSpawner:
    def __init__(self, window=None):
        """
        Editor dialog spawner

        :param window: Window instance
        """
        self.window = window

    def setup(self, id: str = None) -> BaseDialog:
        """
        Setup file editor dialog instance

        :param id: editor id
        :return: BaseDialog instance
        """
        self.window.ui.editor[id] = TextFileEditor(self.window)
        self.window.ui.editor[id].setReadOnly(False)
        self.window.ui.editor[id].setProperty('class', 'code-editor')
        self.window.ui.editor[id].textChanged.connect(
            lambda: self.on_changed(id)
        )

        self.window.ui.nodes['editor.custom.btn.restore'] = QPushButton(trans("action.restore"))
        self.window.ui.nodes['editor.custom.btn.save'] = QPushButton(trans("dialog.editor.btn.save"))
        self.window.ui.nodes['editor.custom.btn.restore'].clicked.connect(
            lambda: self.window.tools.get("editor").restore(id)
        )
        self.window.ui.nodes['editor.custom.btn.save'].clicked.connect(
            lambda: self.window.tools.get("editor").save(id)
        )

        bottom_layout = QHBoxLayout()
        bottom_layout.addWidget(self.window.ui.nodes['editor.custom.btn.restore'])
        bottom_layout.addWidget(self.window.ui.nodes['editor.custom.btn.save'])

        self.window.ui.paths[id] = HelpLabel("")
        self.window.ui.paths[id].setWordWrap(False)

        layout = QVBoxLayout()
        layout.addWidget(self.window.ui.editor[id])
        layout.addWidget(self.window.ui.paths[id])
        layout.addLayout(bottom_layout)

        dialog = EditorFileDialog(self.window)
        dialog.disable_geometry_store = True  # disable geometry store
        dialog.id = id
        dialog.append_layout(layout)
        dialog.setWindowTitle("Text editor")

        return dialog

    def on_changed(self, id: str):
        """
        On content changed event

        :param id: dialog id
        """
        if id in self.window.ui.dialog:
            self.window.ui.dialog[id].update_file_title()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.25 21:00:00                  #
# ================================================== #

from pygpt_net.ui.widget.vision.camera import VideoContainer


class Video:
    def __init__(self, window=None):
        """
        Context list UI

        :param window: Window instance
        """
        self.window = window

    def setup(self) -> VideoContainer:
        """
        Setup video preview
        :return: VideoContainer
        :rtype: VideoContainer
        """
        self.window.ui.nodes['video.preview'] = VideoContainer(self.window)
        self.window.ui.nodes['video.preview'].setVisible(False)

        return self.window.ui.nodes['video.preview']



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.25 12:00:00                  #
# ================================================== #

import json

from PySide6.QtWidgets import QApplication

from pygpt_net.item.ctx import CtxItem


class Event:

    # Events
    AI_NAME = "ai.name"
    AUDIO_INPUT_RECORD_START = "audio.input.record.start"  # start audio input recording
    AUDIO_INPUT_RECORD_STOP = "audio.input.record.stop"  # stop audio input recording
    AUDIO_INPUT_RECORD_TOGGLE = "audio.input.record.toggle"  # toggle audio input recording
    AUDIO_INPUT_TRANSCRIBE = "audio.input.transcribe"  # transcribe audio file
    AUDIO_INPUT_STOP = "audio.input.stop"
    AUDIO_INPUT_TOGGLE = "audio.input.toggle"
    AUDIO_OUTPUT_STOP = "audio.output.stop"
    AUDIO_OUTPUT_TOGGLE = "audio.output.toggle"
    AUDIO_READ_TEXT = "audio.read_text"
    CMD_EXECUTE = "cmd.execute"
    CMD_INLINE = "cmd.inline"
    CMD_SYNTAX = "cmd.syntax"
    CMD_SYNTAX_INLINE = "cmd.syntax.inline"
    CTX_AFTER = "ctx.after"
    CTX_BEFORE = "ctx.before"
    CTX_BEGIN = "ctx.begin"
    CTX_END = "ctx.end"
    CTX_SELECT = "ctx.select"
    DISABLE = "disable"
    ENABLE = "enable"
    FORCE_STOP = "force.stop"
    INPUT_BEFORE = "input.before"
    MODE_BEFORE = "mode.before"
    MODE_SELECT = "mode.select"
    MODEL_BEFORE = "model.before"
    MODEL_SELECT = "model.select"
    PLUGIN_SETTINGS_CHANGED = "plugin.settings.changed"
    PLUGIN_OPTION_GET = "plugin.option.get"
    POST_PROMPT = "post.prompt"
    PRE_PROMPT = "pre.prompt"
    SYSTEM_PROMPT = "system.prompt"
    UI_ATTACHMENTS = "ui.attachments"
    UI_VISION = "ui.vision"
    USER_NAME = "user.name"
    USER_SEND = "user.send"

    def __init__(
            self,
            name: str = None,
            data: dict = None,
            ctx: CtxItem = None
    ):
        """
        Event object class

        :param name: event name
        :param data: event data
        :param ctx: context instance
        """
        self.name = name
        self.data = data
        self.ctx = ctx  # CtxItem
        self.stop = False  # True to stop propagation
        self.internal = False
        # internal event, not called from user
        # internal event is handled synchronously, ctx item has internal flag

    def to_dict(self) -> dict:
        """Dump event to dict"""
        return {
            'name': self.name,
            'data': self.data,
            'ctx': self.ctx.to_dict() if self.ctx else None,
            'stop': self.stop,
            'internal': self.internal,
        }

    def dump(self) -> str:
        """
        Dump event to json string

        :return: JSON string
        """
        try:
            return json.dumps(self.to_dict())
        except Exception as e:
            pass
        return ""

    def __str__(self):
        return self.dump()


class Dispatcher:
    def __init__(self, window=None):
        """
        Event dispatcher

        :param window: Window instance
        """
        self.window = window
        self.nolog_events = ["system.prompt"]
        self.reply_stack = []
        self.reply_ctx = None

    def is_log(self, event: Event) -> bool:
        """
        Check if event can be logged

        :param event: event object
        :return: true if can be logged
        """
        if event.name in self.nolog_events:
            return False
        if not self.is_log_display():
            return False
        data = event.data
        if data is not None and "silent" in data and data["silent"]:
            return False
        return True

    def is_log_display(self) -> bool:
        """
        Check if event can be logged

        :return: true if can be logged
        """
        is_log = False
        if self.window.core.config.has("log.events") \
                and self.window.core.config.get("log.events"):
            is_log = True
        return is_log

    def async_allowed(self, ctx: CtxItem) -> bool:
        """
        Check if async execution are allowed

        :param ctx: context item
        :return: True if async commands are allowed
        """
        disallowed_modes = ["assistant", "agent"]
        if ctx.internal:
            return False
        if self.window.core.config.get("mode") in disallowed_modes:
            return False
        if len(ctx.cmds) > 1:  # if multiple commands then run synchronously
            return False
        return True

    def dispatch(
            self,
            event: Event,
            all: bool = False
    ) -> (list, Event):
        """
        Dispatch event to plugins

        :param event: event to dispatch
        :param all: true if dispatch to all plugins (enabled or not)
        :return: list of affected plugins ids and event object
        """
        if self.is_log(event):
            self.window.core.debug.info("Dispatch event begin: " + event.name)
            if self.window.core.debug.enabled():
                self.window.core.debug.debug("EVENT BEFORE: " + str(event))

        affected = []
        for id in self.window.core.plugins.plugins:
            if self.window.controller.plugins.is_enabled(id) or all:
                if event.stop:
                    break
                if self.window.core.debug.enabled() and self.is_log(event):
                    self.window.core.debug.debug("Apply [{}] to plugin: ".format(event.name) + id)
                self.apply(id, event)
                affected.append(id)

        if self.is_log(event):
            self.window.core.debug.info("Dispatch event end: " + event.name)
            if self.window.core.debug.enabled():
                self.window.core.debug.debug("EVENT AFTER: " + str(event))

        return affected, event

    def apply(
            self,
            id: str,
            event: Event
    ):
        """
        Handle event in plugin with provided id

        :param id: plugin id
        :param event: event object
        """
        if id in self.window.core.plugins.plugins:
            try:
                self.window.core.plugins.plugins[id].handle(event)
            except AttributeError:
                pass

    def reply(self, ctx: CtxItem, flush: bool = False):
        """
        Send reply from plugins to model

        :param ctx: context object
        :param flush: True to flush reply stack
        """
        if ctx is not None:
            self.window.core.debug.info("Reply...")
            if self.window.core.debug.enabled() and self.is_log_display():
                self.window.core.debug.debug("CTX REPLY: " + str(ctx))
            if ctx.reply:
                self.add_reply(ctx)
            if flush or self.window.core.dispatcher.async_allowed(ctx):
                self.flush_reply_stack()

    def add_reply(self, ctx: CtxItem):
        """
        Add reply to stack

        :param ctx: context object
        """
        self.window.core.debug.info("Reply stack (add)...")
        self.reply_stack.append(ctx.results)
        ctx.results = []  # clear results
        self.reply_ctx = ctx

    def flush_reply_stack(self):
        """Flush reply stack"""
        if self.reply_ctx is None or len(self.reply_stack) == 0:
            return
        self.window.core.debug.info("Reply stack (flush)...")
        results = []
        for responses in self.reply_stack:
            for result in responses:
                results.append(result)

        self.window.ui.status("")  # clear status
        if self.reply_ctx.internal:
            if self.window.controller.agent.enabled():
                self.window.controller.agent.add_run()
                self.window.controller.agent.update()
                QApplication.processEvents()

        # prepare data to send as reply
        data = json.dumps(results)
        if len(self.reply_stack) < 2 and self.reply_ctx.extra_ctx and self.window.core.config.get("ctx.use_extra"):
            data = self.reply_ctx.extra_ctx  # if extra content is set, use it as data to send

        prev_ctx = self.window.core.ctx.as_previous(self.reply_ctx)  # copy result to previous ctx and clear current ctx
        self.window.core.ctx.update_item(self.reply_ctx)  # update context in db
        self.window.ui.status('...')
        self.window.controller.chat.input.send(
            text=data,
            force=True,
            reply=True,
            internal=self.reply_ctx.internal,
            prev_ctx=prev_ctx,
        )
        self.clear_reply_stack()

    def clear_reply_stack(self):
        """Clear reply stack"""
        self.window.core.debug.info("Reply stack (clear)...")
        self.reply_ctx = None
        self.reply_stack = []



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.02.23 19:00:00                  #
# ================================================== #

import os
import requests

from .base import BaseProvider


class MSAzureTextToSpeech(BaseProvider):
    def __init__(self, *args, **kwargs):
        """
        Microsoft Azure Text to Speech provider

        :param args: args
        :param kwargs: kwargs
        """
        super(MSAzureTextToSpeech, self).__init__(*args, **kwargs)
        self.plugin = kwargs.get("plugin")
        self.id = "ms_azure"
        self.name = "Microsoft Azure TTS"

    def init_options(self):
        """Initialize options"""
        url_api = {
            "API Key": "https://azure.microsoft.com/en-us/services/cognitive-services/text-to-speech",
        }
        self.plugin.add_option(
            "azure_api_key",
            type="text",
            value="",
            label="Azure API Key",
            tab="ms_azure",
            description="You can obtain your own API key at: "
                        "https://azure.microsoft.com/en-us/services/cognitive-services/text-to-speech/",
            tooltip="Azure API Key",
            secret=True,
            persist=True,
            urls=url_api,
        )
        self.plugin.add_option(
            "azure_region",
            type="text",
            value="eastus",
            label="Azure Region",
            tab="ms_azure",
            description="Specify Azure region, e.g. eastus",
            tooltip="Azure Region",
        )
        self.plugin.add_option(
            "azure_voice_en",
            type="text",
            value="en-US-AriaNeural",
            label="Voice (EN)",
            tab="ms_azure",
            description="Specify voice for English language, e.g. en-US-AriaNeural",
            tooltip="Voice (EN)",
        )
        self.plugin.add_option(
            "azure_voice_pl",
            type="text",
            value="pl-PL-AgnieszkaNeural",
            label="Voice (non-English)",
            tab="ms_azure",
            description="Specify voice for non-English languages, "
                        "e.g. pl-PL-MarekNeural or pl-PL-AgnieszkaNeural",
            tooltip="Voice (PL)",
        )

    def speech(self, text: str) -> str:
        """
        Speech text to audio

        :param text: text to speech
        :return: path to generated audio file or None if audio playback is handled here
        """
        api_key = self.plugin.get_option_value("azure_api_key")
        region = self.plugin.get_option_value("azure_region")
        lang = self.plugin.window.core.config.get('lang')
        output_file = self.plugin.output_file
        path = os.path.join(
            self.plugin.window.core.config.path,
            output_file,
        )
        if lang == "en":
            voice = self.plugin.get_option_value("azure_voice_en")
        else:
            voice = self.plugin.get_option_value("azure_voice_pl")

        url = f"https://{region}.tts.speech.microsoft.com/cognitiveservices/v1"
        headers = {
            "Ocp-Apim-Subscription-Key": api_key,
            "Content-Type": "application/ssml+xml",
            "X-Microsoft-OutputFormat": "audio-16khz-128kbitrate-mono-mp3"
        }
        body = f"<speak version='1.0' xmlns='http://www.w3.org/2001/10/synthesis' " \
               f"xml:lang='en-US'><voice name='{voice}'>{text}</voice></speak>"
        response = requests.post(
            url,
            headers=headers,
            data=body.encode('utf-8'),
        )
        if response.status_code == 200:
            with open(path, "wb") as file:
                file.write(response.content)
            return path

    def is_configured(self) -> bool:
        """
        Check if provider is configured

        :return: True if configured, False otherwise
        """
        api_key = self.plugin.get_option_value("azure_api_key")
        region = self.plugin.get_option_value("azure_region")
        return api_key is not None and api_key != "" and region is not None and region != ""

    def get_config_message(self) -> str:
        """
        Return message to display when provider is not configured

        :return: message
        """
        api_key = self.plugin.get_option_value("azure_api_key")
        region = self.plugin.get_option_value("azure_region")
        if api_key is None or api_key == "":
            return "Azure API KEY is not set. Please set it in plugin settings."
        if region is None or region == "":
            return "Azure Region is not set. Please set it in plugin settings."




#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.31 04:00:00                  #
# ================================================== #

import json
import os
import uuid

from pygpt_net.item.assistant import AssistantStoreItem
from pygpt_net.provider.core.assistant.base import BaseProvider


class JsonFileProvider(BaseProvider):
    def __init__(self, window=None):
        super(JsonFileProvider, self).__init__(window)
        self.window = window
        self.id = "json_file"
        self.type = "assistant_store"
        self.config_file = 'assistants_vector_store.json'

    def create_id(self) -> str:
        """
        Create unique uuid

        :return: uuid
        """
        return str(uuid.uuid4())

    def create(self, store: AssistantStoreItem) -> str:
        """
        Create new and return its ID

        :param store: AssistantStoreItem
        :return: vector store ID
        """
        if store.id is None or store.id == "":
            store.id = self.create_id()
        return store.id

    def load(self) -> dict:
        """
        Load assistants vector store from file

        :return: dict of assistants vector store
        """
        items = {}
        path = os.path.join(self.window.core.config.path, self.config_file)
        try:
            if os.path.exists(path):
                with open(path, 'r', encoding="utf-8") as file:
                    data = json.load(file)
                    if data == "" or data is None or 'items' not in data:
                        return {}
                    # deserialize
                    for id in data['items']:
                        item = data['items'][id]
                        store = AssistantStoreItem()
                        self.deserialize(item, store)
                        items[id] = store
        except Exception as e:
            self.window.core.debug.log(e)
            items = {}
        return items

    def save(self, items: dict):
        """
        Save assistants vector store to file

        :param items: dict of assistants vector store
        """
        try:
            # update store
            path = os.path.join(self.window.core.config.path, self.config_file)
            data = {}
            ary = {}

            # serialize
            for id in items:
                store = items[id]
                ary[id] = self.serialize(store)

            data['__meta__'] = self.window.core.config.append_meta()
            data['items'] = ary
            dump = json.dumps(data, indent=4)
            with open(path, 'w', encoding="utf-8") as f:
                f.write(dump)

        except Exception as e:
            self.window.core.debug.log(e)
            print("Error while saving assistants vector store: {}".format(str(e)))

    def remove(self, id: str):
        """
        Delete by id

        :param id: id
        """
        pass

    def truncate(self):
        """Delete all"""
        pass

    @staticmethod
    def serialize(item: AssistantStoreItem) -> dict:
        """
        Serialize item to dict

        :param item: item to serialize
        :return: serialized item
        """
        return {
            'id': item.id,
            'name': item.name,
            'status': item.status,
            'file_ids': item.file_ids,
        }

    @staticmethod
    def deserialize(data: dict, item: AssistantStoreItem):
        """
        Deserialize item from dict

        :param data: serialized item
        :param item: item to deserialize
        """
        if 'id' in data:
            item.id = data['id']
        if 'name' in data:
            item.name = data['name']
        if 'status' in data:
            item.status = data['status']
        if 'file_ids' in data:
            item.file_ids = data['file_ids']

    def dump(self, item: AssistantStoreItem) -> str:
        """
        Dump to string

        :param item: item to dump
        :return: dumped item as string (json)
        """
        return json.dumps(self.serialize(item))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2023.12.25 21:00:00                  #
# ================================================== #

from PySide6.QtCore import Qt
from PySide6.QtWidgets import QLabel, QHBoxLayout, QWidget


class AudioOutput(QWidget):
    def __init__(self, window=None):
        """
        Settings audio output options

        :param window: main window
        """
        super(AudioOutput, self).__init__(window)
        self.window = window

        # stop button
        # self.stop = QPushButton("STOP")
        # self.stop.setVisible(False)
        # self.stop.setMaximumHeight(40)

        self.status = QLabel("")
        self.layout = QHBoxLayout(self)
        self.layout.addWidget(self.status)
        # self.layout.addWidget(self.stop)
        self.layout.setAlignment(Qt.AlignCenter)
        self.setLayout(self.layout)

        self.setMinimumHeight(40)
        self.setMaximumHeight(40)

    def add_widget(self, widget):
        """
        Add widget to layout

        :param widget: QWidget
        """
        self.layout.addWidget(widget)

    def set_status(self, text):
        """
        Set status text
        :param text: Text to set
        """
        self.status.setText(text)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.17 01:00:00                  #
# ================================================== #

from PySide6.QtWidgets import QLabel, QVBoxLayout, QWidget, QCheckBox, QHBoxLayout

from pygpt_net.ui.widget.element.labels import HelpLabel
from pygpt_net.ui.widget.option.input import DirectoryInput
from pygpt_net.utils import trans


class FilesTab:
    def __init__(self, window=None):
        """
        Tab: Files indexing

        :param window: Window instance
        """
        self.window = window

    def setup(self):
        """
        Setup tab widget
        """
        # path: files
        self.window.ui.nodes["tool.indexer.file.path_file"] = DirectoryInput(
            self.window,
            "tool.indexer",
            "files.file",
            {
                "label": "Path",
                "value": "",
                "extra": {
                    "allow_file": True,
                    "allow_multiple": True,
                },
            }
        )
        self.window.ui.nodes["tool.indexer.file.path_file.label"] = QLabel(trans("tool.indexer.tab.files.path.files"))

        # path: directory
        self.window.ui.nodes["tool.indexer.file.path_dir"] = DirectoryInput(
            self.window,
            "tool.indexer",
            "files.dir",
            {
                "label": "Path",
                "value": "",
                "extra": {
                    "allow_file": False,
                    "allow_multiple": True,
                },
            }
        )
        self.window.ui.nodes["tool.indexer.file.path_dir.label"] = QLabel(trans("tool.indexer.tab.files.path.dir"))

        self.window.ui.nodes["tool.indexer.file.options.recursive"] = QCheckBox(trans("tool.indexer.option.recursive"))
        self.window.ui.nodes["tool.indexer.file.options.recursive"].setChecked(True)
        self.window.ui.nodes["tool.indexer.file.options.replace"] = QCheckBox(trans("tool.indexer.option.replace"))
        self.window.ui.nodes["tool.indexer.file.options.replace"].setChecked(True)
        self.window.ui.nodes["tool.indexer.file.options.clear"] = QCheckBox(trans("tool.indexer.option.clear"))
        self.window.ui.nodes["tool.indexer.file.options.clear"].setChecked(True)

        self.window.ui.nodes["tool.indexer.file.loaders"] = HelpLabel("")

        file_layout = QHBoxLayout()
        file_layout.addWidget(self.window.ui.nodes["tool.indexer.file.path_file.label"])
        file_layout.addWidget(self.window.ui.nodes["tool.indexer.file.path_file"])

        dir_layout = QHBoxLayout()
        dir_layout.addWidget(self.window.ui.nodes["tool.indexer.file.path_dir.label"])
        dir_layout.addWidget(self.window.ui.nodes["tool.indexer.file.path_dir"])

        options_layout = QVBoxLayout()
        options_layout.addWidget(self.window.ui.nodes["tool.indexer.file.options.recursive"])
        options_layout.addWidget(self.window.ui.nodes["tool.indexer.file.options.replace"])
        options_layout.addWidget(self.window.ui.nodes["tool.indexer.file.options.clear"])

        # layout
        self.window.ui.nodes["tool.indexer.file.header.tip"] = HelpLabel(trans("tool.indexer.tab.files.tip"))
        layout = QVBoxLayout()
        layout.addWidget(self.window.ui.nodes["tool.indexer.file.header.tip"])
        layout.addLayout(file_layout)
        layout.addLayout(dir_layout)
        layout.addWidget(self.window.ui.nodes["tool.indexer.file.loaders"])
        layout.addLayout(options_layout)
        layout.addStretch(1)

        self.window.tools.get("indexer").update_tab_files()

        widget = QWidget()
        widget.setLayout(layout)

        return widget



"""
Github repository reader.

Based on: https://github.com/run-llama/llama_index/blob/main/llama-index-integrations/readers/llama-index-readers-github/llama_index/readers/github/repository/base.py

Extended with config in load_data method.
"""

from llama_index.readers.github.repository.base import GithubRepositoryReader as BaseGithubRepositoryReader
from llama_index.readers.github.repository.github_client import GithubClient

from typing import List, Optional
from llama_index.core.schema import Document

class GithubRepositoryReader(BaseGithubRepositoryReader):
    def __init__(
        self,
        token: str,
        use_parser: bool = False,
        verbose: bool = False,
        concurrent_requests: int = 5,
        timeout: Optional[int] = 5,
        retries: int = 0,
        filter_dirs_include: Optional[List[str]] = None,
        filter_dirs_exclude: Optional[List[str]] = None,
        filter_file_ext_include: Optional[List[str]] = None,
        filter_file_ext_exclude: Optional[List[str]] = None,
    ):
        """
        Init reader.

        :param token: github API token
        :param use_parser: use parser
        :param verbose: verbose mode
        :param concurrent_requests: concurrent requests
        :param timeout: timeout
        :param retries: retries
        :param filter_dirs_include: list of directories to include
        :param filter_dirs_exclude: list of directories to exclude
        :param filter_file_ext_include: list of file extensions to include
        :param filter_file_ext_exclude: list of file extensions to exclude
        """
        self._initialized = False
        self._token = token
        self._verbose = verbose

        filter_directories = None
        filter_file_extensions = None
        if filter_dirs_include is not None:
            filter_directories = (filter_dirs_include, self.FilterType.INCLUDE)
        elif filter_dirs_exclude is not None:
            filter_directories = (filter_dirs_exclude, self.FilterType.EXCLUDE)
        if filter_file_ext_include is not None:
            filter_file_extensions = (filter_file_ext_include, self.FilterType.INCLUDE)
        elif filter_file_ext_exclude is not None:
            filter_file_extensions = (filter_file_ext_exclude, self.FilterType.EXCLUDE)

        super().__init__(
            github_client=None,
            owner="",
            repo="",
            use_parser=use_parser,
            verbose=verbose,
            concurrent_requests=concurrent_requests,
            timeout=timeout,
            retries=retries,
            filter_directories=filter_directories,
            filter_file_extensions=filter_file_extensions,
        )

    def _initialize(self):
        """
        Initialize reader.
        """
        self._github_client = GithubClient(github_token=self._token, verbose=self._verbose)

    def load_data(
            self,
            commit_sha: Optional[str] = None,
            branch: Optional[str] = None,
            owner: Optional[str] = None,
            repository: Optional[str] = None,

    ) -> List[Document]:
        """
        Load data from github repository.

        :param commit_sha: commit sha
        :param branch: branch
        :param owner: owner
        :param repository: repository name

        :return: list of documents
        """
        if not self._initialized:
            self._initialize()
            self._initialized = True

        self._owner = owner
        self._repo = repository
        return super().load_data(commit_sha, branch)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.14 20:00:00                  #
# ================================================== #

from PySide6 import QtCore
from PySide6.QtGui import QStandardItemModel, QIcon
from PySide6.QtWidgets import QVBoxLayout, QWidget
from datetime import datetime, timedelta

from pygpt_net.item.ctx import CtxMeta
from pygpt_net.ui.widget.element.button import NewCtxButton
from pygpt_net.ui.widget.element.labels import TitleLabel
from pygpt_net.ui.widget.lists.context import ContextList, Item, GroupItem
from pygpt_net.utils import trans
import pygpt_net.icons_rc


class CtxList:
    def __init__(self, window=None):
        """
        Context list UI

        :param window: Window instance
        """
        self.window = window

    def setup(self) -> QWidget:
        """
        Setup list

        :return: QWidget
        """
        id = 'ctx.list'
        self.window.ui.nodes['ctx.new'] = NewCtxButton(trans('ctx.new'), self.window)
        self.window.ui.nodes[id] = ContextList(self.window, id)
        self.window.ui.nodes[id].selection_locked = self.window.controller.ctx.context_change_locked
        self.window.ui.nodes['ctx.label'] = TitleLabel(trans("ctx.list.label"))

        layout = QVBoxLayout()
        layout.addWidget(self.window.ui.nodes['ctx.new'])
        layout.addWidget(self.window.ui.nodes[id])
        layout.setContentsMargins(0, 0, 0, 0)

        self.window.ui.models[id] = self.create_model(self.window)
        self.window.ui.nodes[id].setModel(self.window.ui.models[id])

        self.window.ui.nodes[id].selectionModel().selectionChanged.connect(
            lambda: self.window.controller.ctx.selection_change()
        )

        widget = QWidget()
        widget.setLayout(layout)
        widget.setContentsMargins(0, 0, 0, 0)

        return widget

    def create_model(self, parent) -> QStandardItemModel:
        """
        Create model

        :param parent: parent widget
        :return: QStandardItemModel
        """
        return QStandardItemModel(0, 1, parent)

    def update(self, id, data):
        """
        Update ctx list

        :param id: ID of the list
        :param data: Data to update
        """
        self.window.ui.nodes[id].backup_selection()
        self.window.ui.models[id].removeRows(0, self.window.ui.models[id].rowCount())

        # get groups
        groups = self.window.core.ctx.get_groups()

        # 1) not grouped on top
        for meta_id in data:
            if data[meta_id].group_id is None or data[meta_id].group_id == 0:
                item = self.build_item(meta_id, data[meta_id])
                self.window.ui.models[id].appendRow(item)

        # 2) grouped items
        for group_id in groups:
            group = groups[group_id]
            c = self.count_in_group(group.id, data)
            if c == 0 and self.window.core.ctx.search_string is not None and self.window.core.ctx.search_string != "":
                continue  # skip empty groups when searching

            suffix = ""
            if c > 0:
                suffix = " (" + str(c) + ")"
            group_name = group.name + suffix
            group_item = GroupItem(QIcon(":/icons/folder_filled.svg"), group_name, group.id)
            for meta_id in data:
                if data[meta_id].group_id != group.id:
                    continue  # skip not in group
                item = self.build_item(meta_id, data[meta_id])
                group_item.appendRow(item)

            self.window.ui.models[id].appendRow(group_item)

            # expand group
            if group.id in self.window.ui.nodes[id].expanded_items:
                self.window.ui.nodes[id].setExpanded(group_item.index(), True)
            else:
                self.window.ui.nodes[id].setExpanded(group_item.index(), False)

        self.window.ui.nodes[id].restore_selection()

    def count_in_group(self, group_id: int, data: dict) -> int:
        """
        Count items in group

        :param group_id: group id
        :param data: context meta data
        :return: int
        """
        count = 0
        for meta_id in data:
            if data[meta_id].group_id == group_id:
                count += 1
        return count

    def build_item(self, id: int, data: CtxMeta) -> Item:
        """
        Build item for list (child)

        :param id: context meta id
        :param data: context meta item
        :return: Item
        """
        dt = self.convert_date(data.updated)
        date_time_str = datetime.fromtimestamp(data.updated).strftime("%Y-%m-%d %H:%M")
        title = data.name
        # truncate to max 40 chars
        if len(title) > 80:
            title = title[:80] + '...'
        name = title.replace("\n", "") + ' (' + dt + ')'
        mode_str = ''
        if data.last_mode is not None:
            mode_str = " ({})".format(trans('mode.' + data.last_mode))
        tooltip_text = "{}: {}{} #{}".format(
            date_time_str,
            data.name,
            mode_str,
            id,
        )
        item = Item(name, id)
        item.id = id
        item.setData(tooltip_text, QtCore.Qt.ToolTipRole)
        if data.important:
            item.setData(data.label + 10, QtCore.Qt.ItemDataRole.UserRole)
        else:
            item.setData(data.label, QtCore.Qt.ItemDataRole.UserRole)
        item.setData(name)
        return item

    def convert_date(self, timestamp: int) -> str:
        """
        Convert timestamp to human readable format

        :param timestamp: timestamp
        :return: string
        """
        today = datetime.today().date()
        yesterday = today - timedelta(days=1)
        date = datetime.fromtimestamp(timestamp).date()

        days_ago = (today - date).days
        weeks_ago = days_ago // 7

        if date == today:
            return trans('dt.today')
        elif date == yesterday:
            return trans('dt.yesterday')
        elif weeks_ago == 1:
            return trans('dt.week')
        elif 1 < weeks_ago < 4:
            return f"{weeks_ago} " + trans('dt.weeks')
        elif days_ago < 30:
            return f"{days_ago} " + trans('dt.days_ago')
        elif 30 <= days_ago < 32:
            return trans('dt.month')
        else:
            return date.strftime("%Y-%m-%d")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.30 04:00:00                  #
# ================================================== #

from openai import OpenAI

from pygpt_net.item.ctx import CtxItem

from .assistants import Assistants
from .chat import Chat
from .completion import Completion
from .image import Image
from .store import Store
from .summarizer import Summarizer
from .vision import Vision


class Gpt:
    def __init__(self, window=None):
        """
        OpenAI API wrapper core

        :param window: Window instance
        """
        self.window = window
        self.assistants = Assistants(window)
        self.chat = Chat(window)
        self.completion = Completion(window)
        self.image = Image(window)
        self.store = Store(window)
        self.summarizer = Summarizer(window)
        self.vision = Vision(window)

    def get_client(self) -> OpenAI:
        """
        Return OpenAI client

        :return: OpenAI client
        """
        args = {
            "api_key": self.window.core.config.get('api_key'),
            "organization": self.window.core.config.get('organization_key'),
        }
        if self.window.core.config.has('api_endpoint'):
            endpoint = self.window.core.config.get('api_endpoint')
            if endpoint:
                args["base_url"] = endpoint
        return OpenAI(**args)

    def call(self, **kwargs) -> bool:
        """
        Call OpenAI API

        :param kwargs: Keyword arguments
        :return: result
        """
        mode = kwargs.get("mode", None)
        prompt = kwargs.get("prompt", "")
        stream = kwargs.get("stream", False)
        model = kwargs.get("model", None)  # model instance
        system_prompt = kwargs.get("system_prompt", "")
        assistant_id = kwargs.get("assistant_id", "")
        tools_outputs = kwargs.get("tools_outputs", [])

        ctx = kwargs.get("ctx", CtxItem())
        ai_name = ctx.output_name
        thread_id = ctx.thread

        # prepare max tokens
        max_tokens = self.window.core.config.get('max_output_tokens')

        model_id = None
        if model is not None:
            model_id = model.id

        # check max output tokens
        if max_tokens > model.tokens:
            max_tokens = model.tokens

        # minimum 1 token is required
        if max_tokens < 1:
            max_tokens = 1

        response = None
        used_tokens = 0
        kwargs['max_tokens'] = max_tokens  # append max output tokens to kwargs

        file_ids = self.window.controller.files.uploaded_ids  # uploaded files IDs

        # get response
        if mode == "completion":
            response = self.completion.send(**kwargs)
            used_tokens = self.completion.get_used_tokens()

        elif mode == "chat":
            response = self.chat.send(**kwargs)
            used_tokens = self.chat.get_used_tokens()

        elif mode == "image":
            return self.image.generate(**kwargs)  # return here, async handled

        elif mode == "vision":
            response = self.vision.send(**kwargs)
            used_tokens = self.vision.get_used_tokens()
            images = self.vision.get_attachments()  # dict -> key: id, value: path
            urls = self.vision.get_urls()  # list

            # store sent images in ctx
            if len(images) > 0:
                ctx.images = self.window.core.filesystem.make_local_list(list(images.values()))
            if len(urls) > 0:
                ctx.images = urls
                ctx.urls = urls

        elif mode == "assistant":
            # check if assistant is already running and has tools outputs, then submit them, async handled
            if ctx.run_id is not None and len(tools_outputs) > 0:
                self.assistants.worker.tools_submit(
                    ctx,
                    model_id,
                    tools_outputs,  # list of tools outputs
                )
            else:
                # if not running, then send msg and create new assistant run, async handled
                self.assistants.worker.msg_send(
                    ctx,
                    thread_id,
                    assistant_id,
                    model_id,
                    file_ids,
                    prompt,
                    system_prompt,
                )
            return True  # if assistant mode then return here, will be handled async

        # if stream
        if stream:
            ctx.stream = response
            ctx.set_output("", ai_name)  # set empty output
            ctx.input_tokens = used_tokens  # get from input tokens calculation
            return True

        if response is None:
            return False

        # check for errors
        if "error" in response:
            print("Error in GPT response: " + str(response["error"]))
            return False

        # get output text from response
        output = ""
        if mode == "completion":
            output = response.choices[0].text.strip()
        elif mode == "chat" or mode == "vision":
            if response.choices[0]:
                if response.choices[0].message.content:
                    output = response.choices[0].message.content.strip()
                elif response.choices[0].message.tool_calls:
                    ctx.tool_calls = self.window.core.command.unpack_tool_calls(
                        response.choices[0].message.tool_calls
                    )

        ctx.set_output(output, ai_name)
        ctx.set_tokens(
            response.usage.prompt_tokens,
            response.usage.completion_tokens,
        )
        return True

    def quick_call(self, **kwargs) -> str:
        """
        Quick call OpenAI API with custom prompt

        :param kwargs: keyword arguments
        :return: response content
        """
        prompt = kwargs.get("prompt", "")
        system_prompt = kwargs.get("system_prompt", None)
        max_tokens = kwargs.get("max_tokens", 500)
        temperature = kwargs.get("temperature", 0.0)
        model = kwargs.get("model", None)
        if model is None:
            model = self.window.core.models.from_defaults()

        client = self.get_client()
        messages = []
        messages.append({"role": "system", "content": system_prompt})
        messages.append({"role": "user", "content": prompt})
        
        try:
            response = client.chat.completions.create(
                messages=messages,
                model=model.id,
                max_tokens=max_tokens,
                temperature=temperature,
                top_p=1.0,
                frequency_penalty=0.0,
                presence_penalty=0.0,
            )
            return response.choices[0].message.content
        except Exception as e:
            self.window.core.debug.log(e)
            print("Error in GPT quick call: " + str(e))

    def stop(self):
        """Stop OpenAI API"""
        pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# ================================================== #
# This file is a part of PYGPT package               #
# Website: https://pygpt.net                         #
# GitHub:  https://github.com/szczyglis-dev/py-gpt   #
# MIT License                                        #
# Created By  : Marcin Szczygliński                  #
# Updated Date: 2024.04.29 16:00:00                  #
# ================================================== #

from PySide6.QtGui import QAction, QIcon
from PySide6.QtWidgets import QMenu

from pygpt_net.ui.widget.lists.base import BaseList
from pygpt_net.utils import trans
import pygpt_net.icons_rc


class AssistantVectorStoreEditorList(BaseList):
    def __init__(self, window=None, id=None):
        """
        Store select menu (in editor)

        :param window: main window
        :param id: parent id
        """
        super(AssistantVectorStoreEditorList, self).__init__(window)
        self.window = window
        self.id = id

    def click(self, val):
        idx = val.row()
        self.window.controller.assistant.store.select(idx)

    def contextMenuEvent(self, event):
        """
        Context menu event

        :param event: context menu event
        """
        actions = {}
        actions['refresh'] = QAction(QIcon(":/icons/reload.svg"),
                                      trans('dialog.assistant.store.menu.current.refresh_store'),
                                      self)
        actions['refresh'].triggered.connect(
            lambda: self.action_refresh(event))

        actions['delete'] = QAction(QIcon(":/icons/delete.svg"), trans('action.delete'), self)
        actions['delete'].triggered.connect(
            lambda: self.action_delete(event))

        actions['clear'] = QAction(QIcon(":/icons/close.svg"), trans('dialog.assistant.store.menu.current.clear_files'), self)
        actions['clear'].triggered.connect(
            lambda: self.action_clear(event))

        actions['truncate'] = QAction(QIcon(":/icons/delete.svg"), trans('dialog.assistant.store.menu.current.truncate_files'),
                                   self)
        actions['truncate'].triggered.connect(
            lambda: self.action_truncate(event))

        menu = QMenu(self)
        menu.addAction(actions['refresh'])
        menu.addAction(actions['delete'])
        menu.addAction(actions['clear'])
        menu.addAction(actions['truncate'])

        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            menu.exec_(event.globalPos())

    def action_delete(self, event):
        """
        Delete action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.assistant.store.delete_by_idx(idx)

    def action_clear(self, event):
        """
        Clear action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.assistant.batch.clear_store_files_by_idx(idx)

    def action_truncate(self, event):
        """
        Truncate action handler

        :param event: mouse event
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.assistant.batch.truncate_store_files_by_idx(idx)

    def action_refresh(self, event):
        """
        Refresh action handler
        """
        item = self.indexAt(event.pos())
        idx = item.row()
        if idx >= 0:
            self.window.controller.assistant.store.refresh_by_idx(idx)

